@@ -268,7 +268,7 @@ var tests = {
       is(chatbar.childNodes.length, 0, "should be no chats left");
       checkPopup();
       is(chatbar.selectedChat, null, "nothing should be selected");
-      is(chatbar.chatboxForURL.size(), 0, "chatboxForURL map should be empty");
+      is(chatbar.chatboxForURL.size, 0, "chatboxForURL map should be empty");
       port.close();
       next();
     });
@@ -364,12 +364,54 @@ var tests = {
       todo(!chatbar.nub.hasAttribute("activity"), "Bug 806266 - nub should no longer have activity");
       // TODO: tests for bug 806266 should arrange to have 2 chats collapsed
       // then open them checking the nub is updated correctly.
-      closeAllChats();
-      port.close();
-      next();
+      // Now we will go and change the embedded iframe in the second chat and
+      // ensure the activity magic still works (ie, check that the unload for
+      // the iframe didn't cause our event handlers to be removed.)
+      ok(!second.hasAttribute("activity"), "second chat should have no activity");
+      let subiframe = iframe2.contentDocument.getElementById("iframe");
+      subiframe.contentWindow.addEventListener("unload", function subunload() {
+        subiframe.contentWindow.removeEventListener("unload", subunload);
+        // ensure all other unload listeners have fired.
+        executeSoon(function() {
+          let evt = iframe2.contentDocument.createEvent("CustomEvent");
+          evt.initCustomEvent("socialChatActivity", true, true, {});
+          iframe2.contentDocument.documentElement.dispatchEvent(evt);
+          ok(second.hasAttribute("activity"), "second chat still has activity after unloading sub-iframe");
+          closeAllChats();
+          port.close();
+          next();
+        })
+      })
+      subiframe.setAttribute("src", "data:text/plain:new location for iframe");
     });
   },
 
+  testOnlyOneCallback: function(next) {
+    let chats = document.getElementById("pinnedchats");
+    let port = Social.provider.getWorkerPort();
+    let numOpened = 0;
+    port.onmessage = function (e) {
+      let topic = e.data.topic;
+      switch (topic) {
+        case "test-init-done":
+          port.postMessage({topic: "test-chatbox-open"});
+          break;
+        case "chatbox-opened":
+          numOpened += 1;
+          port.postMessage({topic: "ping"});
+          break;
+        case "pong":
+          executeSoon(function() {
+            is(numOpened, 1, "only got one open message");
+            chats.removeAll();
+            port.close();
+            next();
+          });
+      }
+    }
+    port.postMessage({topic: "test-init", data: { id: 1 }});
+  },
+
   testSecondTopLevelWindow: function(next) {
     // Bug 817782 - check chats work in new top-level windows.
     const chatUrl = "https://example.com/browser/browser/base/content/test/social_chat.html";
@@ -392,6 +434,50 @@ var tests = {
     port.postMessage({topic: "test-init"});
   },
 
+  testChatWindowChooser: function(next) {
+    // Tests that when a worker creates a chat, it is opened in the correct
+    // window.
+    const chatUrl = "https://example.com/browser/browser/base/content/test/social_chat.html";
+    let chatId = 1;
+    let port = Social.provider.getWorkerPort();
+    port.postMessage({topic: "test-init"});
+
+    function openChat(callback) {
+      port.onmessage = function(e) {
+        if (e.data.topic == "got-chatbox-message")
+          callback();
+      }
+      let url = chatUrl + "?" + (chatId++);
+      port.postMessage({topic: "test-worker-chat", data: url});
+    }
+
+    // open a chat (it will open in the main window)
+    ok(!window.SocialChatBar.hasChats, "first window should start with no chats");
+    openChat(function() {
+      ok(window.SocialChatBar.hasChats, "first window has the chat");
+      // create a second window - although this will be the "most recent",
+      // the fact the first window has a chat open means the first will be targetted.
+      let secondWindow = OpenBrowserWindow();
+      secondWindow.addEventListener("load", function loadListener() {
+        secondWindow.removeEventListener("load", loadListener);
+        ok(!secondWindow.SocialChatBar.hasChats, "second window has no chats");
+        openChat(function() {
+          ok(!secondWindow.SocialChatBar.hasChats, "second window still has no chats");
+          is(window.SocialChatBar.chatbar.childElementCount, 2, "first window now has 2 chats");
+          window.SocialChatBar.chatbar.removeAll();
+          // now open another chat - it should open in the second window (as
+          // it is the "most recent" and no other windows have chats)
+          openChat(function() {
+            ok(!window.SocialChatBar.hasChats, "first window has no chats");
+            ok(secondWindow.SocialChatBar.hasChats, "second window has a chat");
+            secondWindow.close();
+            next();
+          });
+        });
+      })
+    });
+  },
+
   // XXX - note this must be the last test until we restore the login state
   // between tests...
   testCloseOnLogout: function(next) {
