@@ -32,6 +32,8 @@
 #include "nsWrapperCacheInlines.h"
 #include "nsDOMEventTargetHelper.h"
 #include "mozilla/Attributes.h"
+#include "nsDOMClassInfoID.h"
+#include "nsError.h"
 
 using namespace mozilla;
 
@@ -79,10 +81,7 @@ NS_IMPL_CYCLE_COLLECTION_CAN_SKIP_BEGIN(nsEventSource)
   bool isBlack = tmp->IsBlack();
   if (isBlack || tmp->mWaitingForOnStopRequest) {
     if (tmp->mListenerManager) {
-      tmp->mListenerManager->UnmarkGrayJSListeners();
-      NS_UNMARK_LISTENER_WRAPPER(Open)
-      NS_UNMARK_LISTENER_WRAPPER(Message)
-      NS_UNMARK_LISTENER_WRAPPER(Error)
+      tmp->mListenerManager->MarkForCC();
     }
     if (!isBlack && tmp->PreservingWrapper()) {
       xpc_UnmarkGrayObject(tmp->GetWrapperPreserveColor());
@@ -105,23 +104,17 @@ NS_IMPL_CYCLE_COLLECTION_TRACE_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(nsEventSource,
                                                   nsDOMEventTargetHelper)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mSrc)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mNotificationCallbacks)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mLoadGroup)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mChannelEventSink)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mHttpChannel)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mTimer)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mOnOpenListener)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mOnMessageListener)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mOnErrorListener)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mUnicodeDecoder)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mSrc)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mNotificationCallbacks)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mLoadGroup)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mChannelEventSink)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mHttpChannel)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mTimer)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mUnicodeDecoder)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(nsEventSource, nsDOMEventTargetHelper)
   tmp->Close();
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mOnOpenListener)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mOnMessageListener)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mOnErrorListener)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 DOMCI_DATA(EventSource, nsEventSource)
@@ -141,13 +134,14 @@ NS_INTERFACE_MAP_END_INHERITING(nsDOMEventTargetHelper)
 NS_IMPL_ADDREF_INHERITED(nsEventSource, nsDOMEventTargetHelper)
 NS_IMPL_RELEASE_INHERITED(nsEventSource, nsDOMEventTargetHelper)
 
+NS_IMPL_EVENT_HANDLER(nsEventSource, open)
+NS_IMPL_EVENT_HANDLER(nsEventSource, message)
+NS_IMPL_EVENT_HANDLER(nsEventSource, error)
+
 void
 nsEventSource::DisconnectFromOwner()
 {
   nsDOMEventTargetHelper::DisconnectFromOwner();
-  NS_DISCONNECT_EVENT_HANDLER(Open)
-  NS_DISCONNECT_EVENT_HANDLER(Message)
-  NS_DISCONNECT_EVENT_HANDLER(Error)
   Close();
 }
 
@@ -163,7 +157,7 @@ nsEventSource::GetUrl(nsAString& aURL)
 }
 
 NS_IMETHODIMP
-nsEventSource::GetReadyState(PRInt32 *aReadyState)
+nsEventSource::GetReadyState(int32_t *aReadyState)
 {
   NS_ENSURE_ARG_POINTER(aReadyState);
   *aReadyState = mReadyState;
@@ -178,24 +172,6 @@ nsEventSource::GetWithCredentials(bool *aWithCredentials)
   return NS_OK;
 }
 
-#define NS_EVENTSRC_IMPL_DOMEVENTLISTENER(_eventlistenername, _eventlistener)  \
-  NS_IMETHODIMP                                                                \
-  nsEventSource::GetOn##_eventlistenername(nsIDOMEventListener * *aListener)   \
-  {                                                                            \
-    return GetInnerEventListener(_eventlistener, aListener);                   \
-  }                                                                            \
-                                                                               \
-  NS_IMETHODIMP                                                                \
-  nsEventSource::SetOn##_eventlistenername(nsIDOMEventListener * aListener)    \
-  {                                                                            \
-    return RemoveAddEventListener(NS_LITERAL_STRING(#_eventlistenername),      \
-                                  _eventlistener, aListener);                  \
-  }
-
-NS_EVENTSRC_IMPL_DOMEVENTLISTENER(open, mOnOpenListener)
-NS_EVENTSRC_IMPL_DOMEVENTLISTENER(error, mOnErrorListener)
-NS_EVENTSRC_IMPL_DOMEVENTLISTENER(message, mOnMessageListener)
-
 NS_IMETHODIMP
 nsEventSource::Close()
 {
@@ -212,7 +188,7 @@ nsEventSource::Close()
 
   if (mTimer) {
     mTimer->Cancel();
-    mTimer = nsnull;
+    mTimer = nullptr;
   }
 
   ResetConnection();
@@ -223,10 +199,10 @@ nsEventSource::Close()
     delete static_cast<Message*>(mMessagesToDispatch.PopFront());
   }
 
-  mSrc = nsnull;
+  mSrc = nullptr;
   mFrozen = false;
 
-  mUnicodeDecoder = nsnull;
+  mUnicodeDecoder = nullptr;
 
   mReadyState = nsIEventSource::CLOSED;
 
@@ -260,7 +236,7 @@ nsEventSource::Init(nsIPrincipal* aPrincipal,
 
   nsCOMPtr<nsIJSContextStack> stack =
     do_GetService("@mozilla.org/js/xpc/ContextStack;1");
-  JSContext* cx = nsnull;
+  JSContext* cx = nullptr;
   if (stack && NS_SUCCEEDED(stack->Peek(&cx)) && cx) {
     const char *filename;
     if (nsJSUtils::GetCallingLocation(cx, &filename, &mScriptLine)) {
@@ -289,7 +265,7 @@ nsEventSource::Init(nsIPrincipal* aPrincipal,
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIURI> srcURI;
-  rv = NS_NewURI(getter_AddRefs(srcURI), aURL, nsnull, baseURI);
+  rv = NS_NewURI(getter_AddRefs(srcURI), aURL, nullptr, baseURI);
   NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_SYNTAX_ERR);
 
   // we observe when the window freezes and thaws
@@ -307,7 +283,7 @@ nsEventSource::Init(nsIPrincipal* aPrincipal,
   rv = nsContentUtils::GetUTFOrigin(srcURI, origin);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCAutoString spec;
+  nsAutoCString spec;
   rv = srcURI->GetSpec(spec);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -349,7 +325,7 @@ NS_IMETHODIMP
 nsEventSource::Initialize(nsISupports* aOwner,
                           JSContext* aContext,
                           JSObject* aObject,
-                          PRUint32 aArgc,
+                          uint32_t aArgc,
                           jsval* aArgv)
 {
   if (mReadyState != nsIEventSource::CONNECTING || !PrefEnabled() ||
@@ -437,10 +413,10 @@ nsEventSource::Observe(nsISupports* aSubject,
   DebugOnly<nsresult> rv;
   if (strcmp(aTopic, DOM_WINDOW_FROZEN_TOPIC) == 0) {
     rv = Freeze();
-    NS_ASSERTION(rv, "Freeze() failed");
+    NS_ASSERTION(NS_SUCCEEDED(rv), "Freeze() failed");
   } else if (strcmp(aTopic, DOM_WINDOW_THAWED_TOPIC) == 0) {
     rv = Thaw();
-    NS_ASSERTION(rv, "Thaw() failed");
+    NS_ASSERTION(NS_SUCCEEDED(rv), "Thaw() failed");
   } else if (strcmp(aTopic, DOM_WINDOW_DESTROYED_TOPIC) == 0) {
     Close();
   }
@@ -466,7 +442,7 @@ nsEventSource::OnStartRequest(nsIRequest *aRequest,
   rv = httpChannel->GetRequestSucceeded(&requestSucceeded);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCAutoString contentType;
+  nsAutoCString contentType;
   rv = httpChannel->GetContentType(contentType);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -502,9 +478,9 @@ NS_METHOD
 nsEventSource::StreamReaderFunc(nsIInputStream *aInputStream,
                                 void *aClosure,
                                 const char *aFromRawSegment,
-                                PRUint32 aToOffset,
-                                PRUint32 aCount,
-                                PRUint32 *aWriteCount)
+                                uint32_t aToOffset,
+                                uint32_t aCount,
+                                uint32_t *aWriteCount)
 {
   nsEventSource* thisObject = static_cast<nsEventSource*>(aClosure);
   if (!thisObject || !aWriteCount) {
@@ -514,7 +490,7 @@ nsEventSource::StreamReaderFunc(nsIInputStream *aInputStream,
 
   *aWriteCount = 0;
 
-  PRInt32 srcCount, outCount;
+  int32_t srcCount, outCount;
   PRUnichar out[2];
   nsresult rv;
 
@@ -537,7 +513,7 @@ nsEventSource::StreamReaderFunc(nsIInputStream *aInputStream,
       p = p + srcCount + 1;
       thisObject->mUnicodeDecoder->Reset();
     } else {
-      for (PRInt32 i = 0; i < outCount; ++i) {
+      for (int32_t i = 0; i < outCount; ++i) {
         rv = thisObject->ParseCharacter(out[i]);
         NS_ENSURE_SUCCESS(rv, rv);
       }
@@ -561,15 +537,15 @@ NS_IMETHODIMP
 nsEventSource::OnDataAvailable(nsIRequest *aRequest,
                                nsISupports *aContext,
                                nsIInputStream *aInputStream,
-                               PRUint32 aOffset,
-                               PRUint32 aCount)
+                               uint64_t aOffset,
+                               uint32_t aCount)
 {
   NS_ENSURE_ARG_POINTER(aInputStream);
 
   nsresult rv = CheckHealthOfRequestCallback(aRequest);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRUint32 totalRead;
+  uint32_t totalRead;
   return aInputStream->ReadSegments(nsEventSource::StreamReaderFunc, this,
                                     aCount, &totalRead);
 }
@@ -669,11 +645,11 @@ private:
 NS_IMPL_CYCLE_COLLECTION_CLASS(AsyncVerifyRedirectCallbackFwr)
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(AsyncVerifyRedirectCallbackFwr)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR_AMBIGUOUS(mEventSource, nsIEventSource)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mEventSource)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(AsyncVerifyRedirectCallbackFwr)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mEventSource)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK(mEventSource)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(AsyncVerifyRedirectCallbackFwr)
@@ -691,7 +667,7 @@ NS_IMPL_CYCLE_COLLECTING_RELEASE(AsyncVerifyRedirectCallbackFwr)
 NS_IMETHODIMP
 nsEventSource::AsyncOnChannelRedirect(nsIChannel *aOldChannel,
                                       nsIChannel *aNewChannel,
-                                      PRUint32    aFlags,
+                                      uint32_t    aFlags,
                                       nsIAsyncVerifyRedirectCallback *aCallback)
 {
   nsCOMPtr<nsIRequest> aOldRequest = do_QueryInterface(aOldChannel);
@@ -724,8 +700,8 @@ nsEventSource::AsyncOnChannelRedirect(nsIChannel *aOldChannel,
                                                    aNewChannel,
                                                    aFlags, fwd);
     if (NS_FAILED(rv)) {
-      mRedirectCallback = nsnull;
-      mNewRedirectChannel = nsnull;
+      mRedirectCallback = nullptr;
+      mNewRedirectChannel = nullptr;
       mErrorLoadOnRedirect = true;
       DispatchFailConnection();
     }
@@ -757,10 +733,10 @@ nsEventSource::OnRedirectVerifyCallback(nsresult aResult)
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  mNewRedirectChannel = nsnull;
+  mNewRedirectChannel = nullptr;
 
   mRedirectCallback->OnRedirectVerifyCallback(aResult);
-  mRedirectCallback = nsnull;
+  mRedirectCallback = nullptr;
 
   return NS_OK;
 }
@@ -829,7 +805,7 @@ nsEventSource::GetBaseURI(nsIURI **aBaseURI)
 {
   NS_ENSURE_ARG_POINTER(aBaseURI);
 
-  *aBaseURI = nsnull;
+  *aBaseURI = nullptr;
 
   nsCOMPtr<nsIURI> baseURI;
 
@@ -910,8 +886,8 @@ nsEventSource::InitChannelAndRequestEventSource()
   }
 
   nsCOMPtr<nsIChannel> channel;
-  rv = NS_NewChannel(getter_AddRefs(channel), mSrc, nsnull, mLoadGroup,
-                     nsnull, loadFlags, channelPolicy);
+  rv = NS_NewChannel(getter_AddRefs(channel), mSrc, nullptr, mLoadGroup,
+                     nullptr, loadFlags, channelPolicy);
   NS_ENSURE_SUCCESS(rv, rv);
 
   mHttpChannel = do_QueryInterface(channel);
@@ -927,13 +903,13 @@ nsEventSource::InitChannelAndRequestEventSource()
     mHttpChannel->SetNotificationCallbacks(this);
   }
 
-  nsCOMPtr<nsIStreamListener> listener =
-    new nsCORSListenerProxy(this, mPrincipal, mHttpChannel,
-                            mWithCredentials, &rv);
+  nsRefPtr<nsCORSListenerProxy> listener =
+    new nsCORSListenerProxy(this, mPrincipal, mWithCredentials);
+  rv = listener->Init(mHttpChannel);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Start reading from the channel
-  rv = mHttpChannel->AsyncOpen(listener, nsnull);
+  rv = mHttpChannel->AsyncOpen(listener, nullptr);
   if (NS_SUCCEEDED(rv)) {
     mWaitingForOnStopRequest = true;
   }
@@ -964,7 +940,7 @@ nsEventSource::AnnounceConnection()
   }
 
   nsCOMPtr<nsIDOMEvent> event;
-  rv = NS_NewDOMEvent(getter_AddRefs(event), nsnull, nsnull);
+  rv = NS_NewDOMEvent(getter_AddRefs(event), nullptr, nullptr);
   if (NS_FAILED(rv)) {
     NS_WARNING("Failed to create the open event!!!");
     return;
@@ -979,7 +955,7 @@ nsEventSource::AnnounceConnection()
 
   event->SetTrusted(true);
 
-  rv = DispatchDOMEvent(nsnull, event, nsnull, nsnull);
+  rv = DispatchDOMEvent(nullptr, event, nullptr, nullptr);
   if (NS_FAILED(rv)) {
     NS_WARNING("Failed to dispatch the open event!!!");
     return;
@@ -998,12 +974,12 @@ nsEventSource::ResetConnection()
   }
   mLastConvertionResult = NS_OK;
 
-  mHttpChannel = nsnull;
-  mNotificationCallbacks = nsnull;
-  mChannelEventSink = nsnull;
+  mHttpChannel = nullptr;
+  mNotificationCallbacks = nullptr;
+  mChannelEventSink = nullptr;
   mStatus = PARSE_STATE_OFF;
-  mRedirectCallback = nsnull;
-  mNewRedirectChannel = nsnull;
+  mRedirectCallback = nullptr;
+  mNewRedirectChannel = nullptr;
 
   mReadyState = nsIEventSource::CONNECTING;
 
@@ -1034,7 +1010,7 @@ nsEventSource::ReestablishConnection()
   }
 
   nsCOMPtr<nsIDOMEvent> event;
-  rv = NS_NewDOMEvent(getter_AddRefs(event), nsnull, nsnull);
+  rv = NS_NewDOMEvent(getter_AddRefs(event), nullptr, nullptr);
   if (NS_FAILED(rv)) {
     NS_WARNING("Failed to create the error event!!!");
     return;
@@ -1049,7 +1025,7 @@ nsEventSource::ReestablishConnection()
 
   event->SetTrusted(true);
 
-  rv = DispatchDOMEvent(nsnull, event, nsnull, nsnull);
+  rv = DispatchDOMEvent(nullptr, event, nullptr, nullptr);
   if (NS_FAILED(rv)) {
     NS_WARNING("Failed to dispatch the error event!!!");
     return;
@@ -1087,7 +1063,7 @@ nsresult
 nsEventSource::PrintErrorOnConsole(const char *aBundleURI,
                                    const PRUnichar *aError,
                                    const PRUnichar **aFormatStrings,
-                                   PRUint32 aFormatStringsLen)
+                                   uint32_t aFormatStringsLen)
 {
   nsCOMPtr<nsIStringBundleService> bundleService =
     mozilla::services::GetStringBundleService();
@@ -1117,9 +1093,9 @@ nsEventSource::PrintErrorOnConsole(const char *aBundleURI,
   }
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = errObj->InitWithWindowID(message.get(),
-                                mScriptFile.get(),
-                                nsnull,
+  rv = errObj->InitWithWindowID(message,
+                                mScriptFile,
+                                EmptyString(),
                                 mScriptLine, 0,
                                 nsIScriptError::errorFlag,
                                 "Event Source", mInnerWindowID);
@@ -1135,7 +1111,7 @@ nsEventSource::PrintErrorOnConsole(const char *aBundleURI,
 nsresult
 nsEventSource::ConsoleError()
 {
-  nsCAutoString targetSpec;
+  nsAutoCString targetSpec;
   nsresult rv = mSrc->GetSpec(targetSpec);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1190,7 +1166,7 @@ nsEventSource::FailConnection()
   }
 
   nsCOMPtr<nsIDOMEvent> event;
-  rv = NS_NewDOMEvent(getter_AddRefs(event), nsnull, nsnull);
+  rv = NS_NewDOMEvent(getter_AddRefs(event), nullptr, nullptr);
   if (NS_FAILED(rv)) {
     NS_WARNING("Failed to create the error event!!!");
     return;
@@ -1205,7 +1181,7 @@ nsEventSource::FailConnection()
 
   event->SetTrusted(true);
 
-  rv = DispatchDOMEvent(nsnull, event, nsnull, nsnull);
+  rv = DispatchDOMEvent(nullptr, event, nullptr, nullptr);
   if (NS_FAILED(rv)) {
     NS_WARNING("Failed to dispatch the error event!!!");
     return;
@@ -1226,7 +1202,7 @@ nsEventSource::CheckCanRequestSrc(nsIURI* aSrc)
   nsCOMPtr<nsIURI> srcToTest = aSrc ? aSrc : mSrc.get();
   NS_ENSURE_TRUE(srcToTest, false);
 
-  PRUint32 aCheckURIFlags =
+  uint32_t aCheckURIFlags =
     nsIScriptSecurityManager::DISALLOW_INHERIT_PRINCIPAL |
     nsIScriptSecurityManager::DISALLOW_SCRIPT;
 
@@ -1243,22 +1219,22 @@ nsEventSource::CheckCanRequestSrc(nsIURI* aSrc)
     nsContentUtils::GetDocumentFromScriptContext(sc);
 
   // mScriptContext should be initialized because of GetBaseURI() above.
-  // Still need to consider the case that doc is nsnull however.
+  // Still need to consider the case that doc is nullptr however.
   rv = CheckInnerWindowCorrectness();
   NS_ENSURE_SUCCESS(rv, false);
-  PRInt16 shouldLoad = nsIContentPolicy::ACCEPT;
+  int16_t shouldLoad = nsIContentPolicy::ACCEPT;
   rv = NS_CheckContentLoadPolicy(nsIContentPolicy::TYPE_DATAREQUEST,
                                  srcToTest,
                                  mPrincipal,
                                  doc,
                                  NS_LITERAL_CSTRING(TEXT_EVENT_STREAM),
-                                 nsnull,    // extra
+                                 nullptr,    // extra
                                  &shouldLoad,
                                  nsContentUtils::GetContentPolicy(),
                                  nsContentUtils::GetSecurityManager());
   isValidContentLoadPolicy = NS_SUCCEEDED(rv) && NS_CP_ACCEPTED(shouldLoad);
 
-  nsCAutoString targetURIScheme;
+  nsAutoCString targetURIScheme;
   rv = srcToTest->GetScheme(targetURIScheme);
   if (NS_SUCCEEDED(rv)) {
     // We only have the http support for now
@@ -1356,7 +1332,7 @@ nsEventSource::DispatchCurrentMessageEvent()
     message->mLastEventID.Assign(mLastEventID);
   }
 
-  PRInt32 sizeBefore = mMessagesToDispatch.GetSize();
+  int32_t sizeBefore = mMessagesToDispatch.GetSize();
   mMessagesToDispatch.Push(message.forget());
   NS_ENSURE_TRUE(mMessagesToDispatch.GetSize() == sizeBefore + 1,
                  NS_ERROR_OUT_OF_MEMORY);
@@ -1391,13 +1367,13 @@ nsEventSource::DispatchAllMessageEvents()
 
   // Let's play get the JSContext
   nsCOMPtr<nsIScriptGlobalObject> sgo = do_QueryInterface(GetOwner());
-  NS_ENSURE_TRUE(sgo,);
+  NS_ENSURE_TRUE_VOID(sgo);
 
   nsIScriptContext* scriptContext = sgo->GetContext();
-  NS_ENSURE_TRUE(scriptContext,);
+  NS_ENSURE_TRUE_VOID(scriptContext);
 
   JSContext* cx = scriptContext->GetNativeContext();
-  NS_ENSURE_TRUE(cx,);
+  NS_ENSURE_TRUE_VOID(cx);
 
   while (mMessagesToDispatch.GetSize() > 0) {
     nsAutoPtr<Message>
@@ -1411,7 +1387,7 @@ nsEventSource::DispatchAllMessageEvents()
       jsString = JS_NewUCStringCopyN(cx,
                                      message->mData.get(),
                                      message->mData.Length());
-      NS_ENSURE_TRUE(jsString,);
+      NS_ENSURE_TRUE_VOID(jsString);
 
       jsData = STRING_TO_JSVAL(jsString);
     }
@@ -1420,7 +1396,7 @@ nsEventSource::DispatchAllMessageEvents()
     // which does not bubble, is not cancelable, and has no default action
 
     nsCOMPtr<nsIDOMEvent> event;
-    rv = NS_NewDOMMessageEvent(getter_AddRefs(event), nsnull, nsnull);
+    rv = NS_NewDOMMessageEvent(getter_AddRefs(event), nullptr, nullptr);
     if (NS_FAILED(rv)) {
       NS_WARNING("Failed to create the message event!!!");
       return;
@@ -1431,7 +1407,7 @@ nsEventSource::DispatchAllMessageEvents()
                                         false, false,
                                         jsData,
                                         mOrigin,
-                                        message->mLastEventID, nsnull);
+                                        message->mLastEventID, nullptr);
     if (NS_FAILED(rv)) {
       NS_WARNING("Failed to init the message event!!!");
       return;
@@ -1439,7 +1415,7 @@ nsEventSource::DispatchAllMessageEvents()
 
     messageEvent->SetTrusted(true);
 
-    rv = DispatchDOMEvent(nsnull, event, nsnull, nsnull);
+    rv = DispatchDOMEvent(nullptr, event, nullptr, nullptr);
     if (NS_FAILED(rv)) {
       NS_WARNING("Failed to dispatch the message event!!!");
       return;
@@ -1500,8 +1476,8 @@ nsEventSource::SetFieldAndClear()
 
     case PRUnichar('r'):
       if (mLastFieldName.EqualsLiteral("retry")) {
-        PRUint32 newValue=0;
-        PRUint32 i = 0;  // we must ensure that there are only digits
+        uint32_t newValue=0;
+        uint32_t i = 0;  // we must ensure that there are only digits
         bool assign = true;
         for (i = 0; i < mLastFieldValue.Length(); ++i) {
           if (mLastFieldValue.CharAt(i) < (PRUnichar)'0' ||
@@ -1510,8 +1486,8 @@ nsEventSource::SetFieldAndClear()
             break;
           }
           newValue = newValue*10 +
-                     (((PRUint32)mLastFieldValue.CharAt(i))-
-                       ((PRUint32)((PRUnichar)'0')));
+                     (((uint32_t)mLastFieldValue.CharAt(i))-
+                       ((uint32_t)((PRUnichar)'0')));
         }
 
         if (assign) {
