@@ -12,8 +12,10 @@
 #include "jsfriendapi.h"
 #include "nsContentUtils.h"
 #include "nsJSUtils.h"
+#include "AudioSampleFormat.h"
 #include "AudioChannelCommon.h"
 
+using namespace mozilla;
 using namespace mozilla::dom;
 
 nsGenericHTMLElement*
@@ -64,20 +66,6 @@ nsHTMLAudioElement::~nsHTMLAudioElement()
 {
 }
 
-void
-nsHTMLAudioElement::GetItemValueText(nsAString& aValue)
-{
-  // Can't call GetSrc because we don't have a JSContext
-  GetURIAttr(nsGkAtoms::src, nullptr, aValue);
-}
-
-void
-nsHTMLAudioElement::SetItemValueText(const nsAString& aValue)
-{
-  // Can't call SetSrc because we don't have a JSContext
-  SetAttr(kNameSpaceID_None, nsGkAtoms::src, aValue, true);
-}
-
 NS_IMETHODIMP
 nsHTMLAudioElement::Initialize(nsISupports* aOwner, JSContext* aContext,
                                JSObject *aObj, uint32_t argc, jsval *argv)
@@ -126,7 +114,7 @@ nsHTMLAudioElement::MozSetup(uint32_t aChannels, uint32_t aRate)
     mAudioStream->Shutdown();
   }
 
-  mAudioStream = nsAudioStream::AllocateStream();
+  mAudioStream = AudioStream::AllocateStream();
   nsresult rv = mAudioStream->Init(aChannels, aRate, mAudioChannelType);
   if (NS_FAILED(rv)) {
     mAudioStream->Shutdown();
@@ -155,7 +143,7 @@ nsHTMLAudioElement::MozWriteAudio(const JS::Value& aData, JSContext* aCx, uint32
   JSObject* tsrc = NULL;
 
   // Allow either Float32Array or plain JS Array
-  if (JS_IsFloat32Array(darray, aCx)) {
+  if (JS_IsFloat32Array(darray)) {
     tsrc = darray;
   } else if (JS_IsArrayObject(aCx, darray)) {
     JSObject* nobj = JS_NewFloat32ArrayFromArray(aCx, darray);
@@ -168,7 +156,7 @@ nsHTMLAudioElement::MozWriteAudio(const JS::Value& aData, JSContext* aCx, uint32
   }
   tvr.setObject(tsrc);
 
-  uint32_t dataLength = JS_GetTypedArrayLength(tsrc, aCx);
+  uint32_t dataLength = JS_GetTypedArrayLength(tsrc);
 
   // Make sure that we are going to write the correct amount of data based
   // on number of channels.
@@ -179,29 +167,14 @@ nsHTMLAudioElement::MozWriteAudio(const JS::Value& aData, JSContext* aCx, uint32
   // Don't write more than can be written without blocking.
   uint32_t writeLen = NS_MIN(mAudioStream->Available(), dataLength / mChannels);
 
-  float* frames = JS_GetFloat32ArrayData(tsrc, aCx);
-#ifdef MOZ_SAMPLE_TYPE_S16
+  float* frames = JS_GetFloat32ArrayData(tsrc);
   // Convert the samples back to integers as we are using fixed point audio in
-  // the nsAudioStream.
-  nsAutoArrayPtr<short> shortsArray(new short[writeLen * mChannels]);
-  // Hard clip the samples.
-  for (uint32_t i = 0; i <  writeLen * mChannels; ++i) {
-    float scaled_value = floorf(0.5 + 32768 * frames[i]);
-    if (frames[i] < 0.0) {
-      shortsArray[i] = (scaled_value < -32768.0) ?
-        -32768 :
-        short(scaled_value);
-    } else {
-      shortsArray[i] = (scaled_value > 32767.0) ?
-        32767 :
-        short(scaled_value);
-    }
-  }
-  nsresult rv = mAudioStream->Write(shortsArray, writeLen);
-#else
-  nsresult rv = mAudioStream->Write(frames, writeLen);
-#endif
-
+  // the AudioStream.
+  // This could be optimized to avoid allocation and memcpy when
+  // AudioDataValue is 'float', but it's not worth it for this deprecated API.
+  nsAutoArrayPtr<AudioDataValue> audioData(new AudioDataValue[writeLen * mChannels]);
+  ConvertAudioSamples(frames, audioData.get(), writeLen * mChannels);
+  nsresult rv = mAudioStream->Write(audioData.get(), writeLen);
 
   if (NS_FAILED(rv)) {
     return rv;
