@@ -10,7 +10,6 @@
 #include "nsIDocument.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMDocumentType.h"
-#include "nsIDOMDOMImplementation.h"
 #include "nsIContent.h"
 #include "nsIURI.h"
 #include "nsNetUtil.h"
@@ -35,12 +34,10 @@
 #include "prlog.h"
 #include "prmem.h"
 #include "nsRect.h"
-#include "nsGenericElement.h"
 #include "nsIWebNavigation.h"
 #include "nsIScriptElement.h"
 #include "nsScriptLoader.h"
 #include "nsStyleLinkElement.h"
-#include "nsIImageLoadingContent.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
 #include "nsICookieService.h"
@@ -52,7 +49,7 @@
 #include "nsContentCreatorFunctions.h"
 #include "nsIContentPolicy.h"
 #include "nsContentPolicyUtils.h"
-#include "nsContentErrors.h"
+#include "nsError.h"
 #include "nsIDOMProcessingInstruction.h"
 #include "nsNodeUtils.h"
 #include "nsIScriptGlobalObject.h"
@@ -79,12 +76,12 @@ NS_NewXMLContentSink(nsIXMLContentSink** aResult,
                      nsISupports* aContainer,
                      nsIChannel* aChannel)
 {
-  NS_PRECONDITION(nsnull != aResult, "null ptr");
-  if (nsnull == aResult) {
+  NS_PRECONDITION(nullptr != aResult, "null ptr");
+  if (nullptr == aResult) {
     return NS_ERROR_NULL_POINTER;
   }
   nsXMLContentSink* it = new nsXMLContentSink();
-  if (nsnull == it) {
+  if (nullptr == it) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
   
@@ -103,7 +100,6 @@ nsXMLContentSink::nsXMLContentSink()
 
 nsXMLContentSink::~nsXMLContentSink()
 {
-  NS_IF_RELEASE(mDocElement);
   if (mText) {
     PR_Free(mText);  //  Doesn't null out, unlike PR_FREEIF
   }
@@ -126,7 +122,7 @@ nsXMLContentSink::Init(nsIDocument* aDoc,
   }
   
   mState = eXMLContentSinkState_InProlog;
-  mDocElement = nsnull;
+  mDocElement = nullptr;
 
   return NS_OK;
 }
@@ -145,9 +141,9 @@ NS_IMPL_CYCLE_COLLECTION_CLASS(nsXMLContentSink)
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(nsXMLContentSink,
                                                   nsContentSink)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mCurrentHead)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_RAWPTR(mDocElement)
-  for (PRUint32 i = 0, count = tmp->mContentStack.Length(); i < count; i++) {
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mCurrentHead)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mDocElement)
+  for (uint32_t i = 0, count = tmp->mContentStack.Length(); i < count; i++) {
     const StackNode& node = tmp->mContentStack.ElementAt(i);
     cb.NoteXPCOMChild(node.mContent);
   }
@@ -170,7 +166,7 @@ nsXMLContentSink::WillBuildModel(nsDTDMode aDTDMode)
 
   // Check for correct load-command for maybe prettyprinting
   if (mPrettyPrintXML) {
-    nsCAutoString command;
+    nsAutoCString command;
     GetParser()->GetCommand(command);
     if (!command.EqualsLiteral("view")) {
       mPrettyPrintXML = false;
@@ -262,6 +258,15 @@ CheckXSLTParamPI(nsIDOMProcessingInstruction* aPi,
 NS_IMETHODIMP
 nsXMLContentSink::DidBuildModel(bool aTerminated)
 {
+  if (!mParser) {
+    // If mParser is null, this parse has already been terminated and must
+    // not been terminated again. However, nsDocument may still think that
+    // the parse has not been terminated and call back into here in the case
+    // where the XML parser has finished but the XSLT transform associated
+    // with the document has not.
+    return NS_OK;
+  }
+
   DidBuildModelImpl(aTerminated);
 
   if (mXSLTProcessor) {
@@ -287,7 +292,7 @@ nsXMLContentSink::DidBuildModel(bool aTerminated)
     mXSLTProcessor->SetSourceContentModel(currentDOMDoc);
     // Since the processor now holds a reference to us we drop our reference
     // to it to avoid owning cycles
-    mXSLTProcessor = nsnull;
+    mXSLTProcessor = nullptr;
   }
   else {
     // Kick off layout for non-XSLT transformed documents.
@@ -445,14 +450,14 @@ nsXMLContentSink::SetParser(nsParserBase* aParser)
 }
 
 nsresult
-nsXMLContentSink::CreateElement(const PRUnichar** aAtts, PRUint32 aAttsCount,
-                                nsINodeInfo* aNodeInfo, PRUint32 aLineNumber,
+nsXMLContentSink::CreateElement(const PRUnichar** aAtts, uint32_t aAttsCount,
+                                nsINodeInfo* aNodeInfo, uint32_t aLineNumber,
                                 nsIContent** aResult, bool* aAppendContent,
                                 FromParser aFromParser)
 {
   NS_ASSERTION(aNodeInfo, "can't create element without nodeinfo");
 
-  *aResult = nsnull;
+  *aResult = nullptr;
   *aAppendContent = true;
   nsresult rv = NS_OK;
 
@@ -529,9 +534,6 @@ nsXMLContentSink::CloseElement(nsIContent* aContent)
 #endif
         nodeInfo->NameAtom() == nsGkAtoms::object ||
         nodeInfo->NameAtom() == nsGkAtoms::applet))
-#ifdef MOZ_XTF
-      || nodeInfo->NamespaceID() > kNameSpaceID_LastBuiltin
-#endif
       || nodeInfo->NameAtom() == nsGkAtoms::title
       ) {
     aContent->DoneAddingChildren(HaveNotifiedForCurrentContent());
@@ -590,7 +592,7 @@ nsXMLContentSink::CloseElement(nsIContent* aContent)
       ssle->SetEnableUpdates(true);
       bool willNotify;
       bool isAlternate;
-      rv = ssle->UpdateStyleSheet(mRunsToCompletion ? nsnull : this,
+      rv = ssle->UpdateStyleSheet(mRunsToCompletion ? nullptr : this,
                                   &willNotify,
                                   &isAlternate);
       if (NS_SUCCEEDED(rv) && willNotify && !isAlternate && !mRunsToCompletion) {
@@ -604,7 +606,7 @@ nsXMLContentSink::CloseElement(nsIContent* aContent)
       nsAutoString relVal;
       aContent->GetAttr(kNameSpaceID_None, nsGkAtoms::rel, relVal);
       if (!relVal.IsEmpty()) {
-        PRUint32 linkTypes = nsStyleLinkElement::ParseLinkTypes(relVal);
+        uint32_t linkTypes = nsStyleLinkElement::ParseLinkTypes(relVal);
         bool hasPrefetch = linkTypes & PREFETCH;
         if (hasPrefetch || (linkTypes & NEXT)) {
           nsAutoString hrefVal;
@@ -689,7 +691,7 @@ nsXMLContentSink::ProcessStyleLink(nsIContent* aElement,
   nsresult rv = NS_OK;
   mPrettyPrintXML = false;
 
-  nsCAutoString cmd;
+  nsAutoCString cmd;
   if (mParser)
     GetParser()->GetCommand(cmd);
   if (cmd.EqualsASCII(kLoadAsData))
@@ -709,7 +711,7 @@ nsXMLContentSink::ProcessStyleLink(nsIContent* aElement,
       return NS_OK;
 
     nsCOMPtr<nsIURI> url;
-    rv = NS_NewURI(getter_AddRefs(url), aHref, nsnull,
+    rv = NS_NewURI(getter_AddRefs(url), aHref, nullptr,
                    mDocument->GetDocBaseURI());
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -721,13 +723,13 @@ nsXMLContentSink::ProcessStyleLink(nsIContent* aElement,
     NS_ENSURE_SUCCESS(rv, NS_OK);
 
     // Do content policy check
-    PRInt16 decision = nsIContentPolicy::ACCEPT;
+    int16_t decision = nsIContentPolicy::ACCEPT;
     rv = NS_CheckContentLoadPolicy(nsIContentPolicy::TYPE_STYLESHEET,
                                    url,
                                    mDocument->NodePrincipal(),
                                    aElement,
                                    type,
-                                   nsnull,
+                                   nullptr,
                                    &decision,
                                    nsContentUtils::GetContentPolicy(),
                                    nsContentUtils::GetSecurityManager());
@@ -782,7 +784,7 @@ nsXMLContentSink::FlushText(bool aReleaseTextNode)
     if (mLastTextNode) {
       if ((mLastTextNodeSize + mTextLength) > mTextSize && !mXSLTProcessor) {
         mLastTextNodeSize = 0;
-        mLastTextNode = nsnull;
+        mLastTextNode = nullptr;
         FlushText(aReleaseTextNode);
       } else {
         bool notify = HaveNotifiedForCurrentContent();
@@ -820,7 +822,7 @@ nsXMLContentSink::FlushText(bool aReleaseTextNode)
 
   if (aReleaseTextNode) {
     mLastTextNodeSize = 0;
-    mLastTextNode = nsnull;
+    mLastTextNode = nullptr;
   }
   
   return rv;
@@ -830,7 +832,7 @@ nsIContent*
 nsXMLContentSink::GetCurrentContent()
 {
   if (mContentStack.Length() == 0) {
-    return nsnull;
+    return nullptr;
   }
   return GetCurrentStackNode()->mContent;
 }
@@ -838,8 +840,8 @@ nsXMLContentSink::GetCurrentContent()
 StackNode*
 nsXMLContentSink::GetCurrentStackNode()
 {
-  PRInt32 count = mContentStack.Length();
-  return count != 0 ? &mContentStack[count-1] : nsnull;
+  int32_t count = mContentStack.Length();
+  return count != 0 ? &mContentStack[count-1] : nullptr;
 }
 
 
@@ -858,7 +860,7 @@ nsXMLContentSink::PushContent(nsIContent *aContent)
 void
 nsXMLContentSink::PopContent()
 {
-  PRInt32 count = mContentStack.Length();
+  int32_t count = mContentStack.Length();
 
   if (count == 0) {
     NS_WARNING("Popping empty stack");
@@ -871,7 +873,7 @@ nsXMLContentSink::PopContent()
 bool
 nsXMLContentSink::HaveNotifiedForCurrentContent() const
 {
-  PRUint32 stackLength = mContentStack.Length();
+  uint32_t stackLength = mContentStack.Length();
   if (stackLength) {
     const StackNode& stackNode = mContentStack[stackLength - 1];
     nsIContent* parent = stackNode.mContent;
@@ -894,7 +896,7 @@ nsXMLContentSink::MaybeStartLayout(bool aIgnorePendingSheets)
 ////////////////////////////////////////////////////////////////////////
 
 bool
-nsXMLContentSink::SetDocElement(PRInt32 aNameSpaceID,
+nsXMLContentSink::SetDocElement(int32_t aNameSpaceID,
                                 nsIAtom* aTagName,
                                 nsIContent *aContent)
 {
@@ -920,7 +922,6 @@ nsXMLContentSink::SetDocElement(PRInt32 aNameSpaceID,
   }
 
   mDocElement = aContent;
-  NS_ADDREF(mDocElement);
   nsresult rv = mDocument->AppendChildTo(mDocElement, NotifyForDocElement());
   if (NS_FAILED(rv)) {
     // If we return false here, the caller will bail out because it won't
@@ -939,9 +940,9 @@ nsXMLContentSink::SetDocElement(PRInt32 aNameSpaceID,
 NS_IMETHODIMP
 nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
                                      const PRUnichar **aAtts,
-                                     PRUint32 aAttsCount,
-                                     PRInt32 aIndex,
-                                     PRUint32 aLineNumber)
+                                     uint32_t aAttsCount,
+                                     int32_t aIndex,
+                                     uint32_t aLineNumber)
 {
   return HandleStartElement(aName, aAtts, aAttsCount, aIndex, aLineNumber,
                             true);
@@ -950,9 +951,9 @@ nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
 nsresult
 nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
                                      const PRUnichar **aAtts,
-                                     PRUint32 aAttsCount,
-                                     PRInt32 aIndex,
-                                     PRUint32 aLineNumber,
+                                     uint32_t aAttsCount,
+                                     int32_t aIndex,
+                                     uint32_t aLineNumber,
                                      bool aInterruptable)
 {
   NS_PRECONDITION(aIndex >= -1, "Bogus aIndex");
@@ -974,7 +975,7 @@ nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
 
   mState = eXMLContentSinkState_InDocumentElement;
 
-  PRInt32 nameSpaceID;
+  int32_t nameSpaceID;
   nsCOMPtr<nsIAtom> prefix, localName;
   nsContentUtils::SplitExpatName(aName, getter_AddRefs(prefix),
                                  getter_AddRefs(localName), &nameSpaceID);
@@ -1012,11 +1013,6 @@ nsXMLContentSink::HandleStartElement(const PRUnichar *aName,
       nodeInfo->SetIDAttributeAtom(IDAttr);
     }
   }
-  
-#ifdef MOZ_XTF
-  if (nameSpaceID > kNameSpaceID_LastBuiltin)
-    content->BeginAddingChildren();
-#endif
 
   // Set the attributes on the new content element
   result = AddAttributes(aAtts, content);
@@ -1092,14 +1088,14 @@ nsXMLContentSink::HandleEndElement(const PRUnichar *aName,
 
   nsCOMPtr<nsIContent> content;
   sn->mContent.swap(content);
-  PRUint32 numFlushed = sn->mNumFlushed;
+  uint32_t numFlushed = sn->mNumFlushed;
 
   PopContent();
   NS_ASSERTION(content, "failed to pop content");
 #ifdef DEBUG
   // Check that we're closing the right thing
   nsCOMPtr<nsIAtom> debugNameSpacePrefix, debugTagAtom;
-  PRInt32 debugNameSpaceID;
+  int32_t debugNameSpaceID;
   nsContentUtils::SplitExpatName(aName, getter_AddRefs(debugNameSpacePrefix),
                                  getter_AddRefs(debugTagAtom),
                                  &debugNameSpaceID);
@@ -1110,7 +1106,7 @@ nsXMLContentSink::HandleEndElement(const PRUnichar *aName,
   result = CloseElement(content);
 
   if (mCurrentHead == content) {
-    mCurrentHead = nsnull;
+    mCurrentHead = nullptr;
   }
   
   if (mDocElement == content) {
@@ -1122,7 +1118,7 @@ nsXMLContentSink::HandleEndElement(const PRUnichar *aName,
     MaybeStartLayout(false);
   }
 
-  PRInt32 stackLen = mContentStack.Length();
+  int32_t stackLen = mContentStack.Length();
   if (mNotifyLevel >= stackLen) {
     if (numFlushed < content->GetChildCount()) {
     	  NotifyAppend(content, numFlushed);
@@ -1161,7 +1157,7 @@ nsXMLContentSink::HandleComment(const PRUnichar *aName)
 
 NS_IMETHODIMP 
 nsXMLContentSink::HandleCDataSection(const PRUnichar *aData, 
-                                     PRUint32 aLength)
+                                     uint32_t aLength)
 {
   // XSLT doesn't differentiate between text and cdata and wants adjacent
   // textnodes merged, so add as text.
@@ -1215,7 +1211,7 @@ nsXMLContentSink::HandleDoctypeDecl(const nsAString & aSubset,
       mCSSLoader->LoadSheetSync(uri, true, true, getter_AddRefs(sheet));
 
 #ifdef DEBUG
-      nsCAutoString uriStr;
+      nsAutoCString uriStr;
       uri->GetSpec(uriStr);
       printf("Loading catalog stylesheet: %s ... %s\n", uriStr.get(), sheet.get() ? "Done" : "Failed");
 #endif
@@ -1237,13 +1233,13 @@ nsXMLContentSink::HandleDoctypeDecl(const nsAString & aSubset,
 
 NS_IMETHODIMP
 nsXMLContentSink::HandleCharacterData(const PRUnichar *aData, 
-                                      PRUint32 aLength)
+                                      uint32_t aLength)
 {
   return HandleCharacterData(aData, aLength, true);
 }
 
 nsresult
-nsXMLContentSink::HandleCharacterData(const PRUnichar *aData, PRUint32 aLength,
+nsXMLContentSink::HandleCharacterData(const PRUnichar *aData, uint32_t aLength,
                                       bool aInterruptable)
 {
   nsresult rv = NS_OK;
@@ -1286,7 +1282,7 @@ nsXMLContentSink::HandleProcessingInstruction(const PRUnichar *aTarget,
     ssle->SetEnableUpdates(true);
     bool willNotify;
     bool isAlternate;
-    rv = ssle->UpdateStyleSheet(mRunsToCompletion ? nsnull : this,
+    rv = ssle->UpdateStyleSheet(mRunsToCompletion ? nullptr : this,
                                 &willNotify,
                                 &isAlternate);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -1353,7 +1349,7 @@ nsXMLContentSink::ParsePIData(const nsString &aData, nsString &aHref,
 NS_IMETHODIMP
 nsXMLContentSink::HandleXMLDeclaration(const PRUnichar *aVersion,
                                        const PRUnichar *aEncoding,
-                                       PRInt32 aStandalone)
+                                       int32_t aStandalone)
 {
   mDocument->SetXMLDeclaration(aVersion, aEncoding, aStandalone);
 
@@ -1394,7 +1390,7 @@ nsXMLContentSink::ReportError(const PRUnichar* aErrorText,
       node->RemoveChild(child, getter_AddRefs(dummy));
     }
   }
-  NS_IF_RELEASE(mDocElement); 
+  mDocElement = nullptr;
 
   // Clear any buffered-up text we have.  It's enough to set the length to 0.
   // The buffer itself is allocated when we're created and deleted in our
@@ -1404,7 +1400,7 @@ nsXMLContentSink::ReportError(const PRUnichar* aErrorText,
   if (mXSLTProcessor) {
     // Get rid of the XSLT processor.
     mXSLTProcessor->CancelLoads();
-    mXSLTProcessor = nsnull;
+    mXSLTProcessor = nullptr;
   }
 
   // release the nodes on stack
@@ -1424,7 +1420,7 @@ nsXMLContentSink::ReportError(const PRUnichar* aErrorText,
   parsererror.Append((PRUnichar)0xFFFF);
   parsererror.AppendLiteral("parsererror");
   
-  rv = HandleStartElement(parsererror.get(), noAtts, 0, -1, (PRUint32)-1,
+  rv = HandleStartElement(parsererror.get(), noAtts, 0, -1, (uint32_t)-1,
                           false);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1435,7 +1431,7 @@ nsXMLContentSink::ReportError(const PRUnichar* aErrorText,
   sourcetext.Append((PRUnichar)0xFFFF);
   sourcetext.AppendLiteral("sourcetext");
 
-  rv = HandleStartElement(sourcetext.get(), noAtts, 0, -1, (PRUint32)-1,
+  rv = HandleStartElement(sourcetext.get(), noAtts, 0, -1, (uint32_t)-1,
                           false);
   NS_ENSURE_SUCCESS(rv, rv);
   
@@ -1460,7 +1456,7 @@ nsXMLContentSink::AddAttributes(const PRUnichar** aAtts,
   // Add tag attributes to the content attributes
   nsCOMPtr<nsIAtom> prefix, localName;
   while (*aAtts) {
-    PRInt32 nameSpaceID;
+    int32_t nameSpaceID;
     nsContentUtils::SplitExpatName(aAtts[0], getter_AddRefs(prefix),
                                    getter_AddRefs(localName), &nameSpaceID);
 
@@ -1477,21 +1473,21 @@ nsXMLContentSink::AddAttributes(const PRUnichar** aAtts,
 
 nsresult
 nsXMLContentSink::AddText(const PRUnichar* aText, 
-                          PRInt32 aLength)
+                          int32_t aLength)
 {
   // Create buffer when we first need it
   if (0 == mTextSize) {
     mText = (PRUnichar *) PR_MALLOC(sizeof(PRUnichar) * NS_ACCUMULATION_BUFFER_SIZE);
-    if (nsnull == mText) {
+    if (nullptr == mText) {
       return NS_ERROR_OUT_OF_MEMORY;
     }
     mTextSize = NS_ACCUMULATION_BUFFER_SIZE;
   }
 
   // Copy data from string into our buffer; flush buffer when it fills up
-  PRInt32 offset = 0;
+  int32_t offset = 0;
   while (0 != aLength) {
-    PRInt32 amount = mTextSize - mTextLength;
+    int32_t amount = mTextSize - mTextLength;
     if (0 == amount) {
       // XSLT wants adjacent textnodes merged.
       if (mConstrainSize && !mXSLTProcessor) {
@@ -1505,7 +1501,7 @@ nsXMLContentSink::AddText(const PRUnichar* aText,
       else {
         mTextSize += aLength;
         mText = (PRUnichar *) PR_REALLOC(mText, sizeof(PRUnichar) * mTextSize);
-        if (nsnull == mText) {
+        if (nullptr == mText) {
           mTextSize = 0;
 
           return NS_ERROR_OUT_OF_MEMORY;
@@ -1564,7 +1560,7 @@ nsXMLContentSink::FlushTags()
 {
   mDeferredFlushTags = false;
   bool oldBeganUpdate = mBeganUpdate;
-  PRUint32 oldUpdates = mUpdatesInNotification;
+  uint32_t oldUpdates = mUpdatesInNotification;
 
   mUpdatesInNotification = 0;
   ++mInNotification;
@@ -1580,10 +1576,10 @@ nsXMLContentSink::FlushTags()
     // a notification from the node that is closest to the root of
     // tree for any content that has been added.
 
-    PRInt32 stackPos;
-    PRInt32 stackLen = mContentStack.Length();
+    int32_t stackPos;
+    int32_t stackLen = mContentStack.Length();
     bool flushed = false;
-    PRUint32 childCount;
+    uint32_t childCount;
     nsIContent* content;
 
     for (stackPos = 0; stackPos < stackLen; ++stackPos) {
@@ -1622,8 +1618,8 @@ nsXMLContentSink::UpdateChildCounts()
   // have been generated for it and we should make sure that no
   // further reflows occur.  Note that we have to include stackPos == 0
   // to properly notify on kids of <html>.
-  PRInt32 stackLen = mContentStack.Length();
-  PRInt32 stackPos = stackLen - 1;
+  int32_t stackLen = mContentStack.Length();
+  int32_t stackPos = stackLen - 1;
   while (stackPos >= 0) {
     StackNode & node = mContentStack[stackPos];
     node.mNumFlushed = node.mContent->GetChildCount();
