@@ -17,6 +17,8 @@ using namespace js;
 using namespace js::mjit;
 using namespace JSC;
 
+using mozilla::DebugOnly;
+
 typedef JSC::MacroAssembler::FPRegisterID FPRegisterID;
 
 CompileStatus
@@ -320,8 +322,8 @@ mjit::Compiler::compileGetChar(FrameEntry *thisValue, FrameEntry *arg, GetCharMo
     masm.move(reg1, reg2);
 
     /* Slow path if string is a rope */
-    masm.andPtr(ImmPtr((void *)JSString::ROPE_BIT), reg1);
-    Jump isRope = masm.branchTestPtr(Assembler::NonZero, reg1);
+    masm.andPtr(ImmPtr((void *)JSString::FLAGS_MASK), reg1);
+    Jump isRope = masm.branchTestPtr(Assembler::Zero, reg1);
     stubcc.linkExit(isRope, Uses(3));
 
     /* Slow path if out-of-range. */
@@ -473,7 +475,7 @@ mjit::Compiler::compileArrayPopShift(FrameEntry *thisValue, bool isPacked, bool
 
 #ifdef JSGC_INCREMENTAL_MJ
     /* Write barrier. */
-    if (cx->compartment->needsBarrier())
+    if (cx->compartment->compileBarriers())
         return Compile_InlineAbort;
 #endif
 
@@ -615,9 +617,7 @@ mjit::Compiler::compileArrayConcat(types::TypeSet *thisTypes, types::TypeSet *ar
      * so check that type information already reflects possible side effects of
      * this call.
      */
-    thisTypes->addFreeze(cx);
-    argTypes->addFreeze(cx);
-    types::TypeSet *thisElemTypes = thisType->getProperty(cx, JSID_VOID, false);
+    types::HeapTypeSet *thisElemTypes = thisType->getProperty(cx, JSID_VOID, false);
     if (!thisElemTypes)
         return Compile_Error;
     if (!pushedTypeSet(0)->hasType(types::Type::ObjectType(thisType)))
@@ -628,7 +628,7 @@ mjit::Compiler::compileArrayConcat(types::TypeSet *thisTypes, types::TypeSet *ar
         types::TypeObject *argType = argTypes->getTypeObject(i);
         if (!argType)
             continue;
-        types::TypeSet *elemTypes = argType->getProperty(cx, JSID_VOID, false);
+        types::HeapTypeSet *elemTypes = argType->getProperty(cx, JSID_VOID, false);
         if (!elemTypes)
             return Compile_Error;
         if (!elemTypes->knownSubset(cx, thisElemTypes))
@@ -705,6 +705,7 @@ mjit::Compiler::compileArrayWithLength(uint32_t argc)
             return Compile_InlineAbort;
     }
 
+    RootedScript script(cx, script_);
     types::TypeObject *type = types::TypeScript::InitObject(cx, script, PC, JSProto_Array);
     if (!type)
         return Compile_Error;
@@ -746,6 +747,7 @@ mjit::Compiler::compileArrayWithArgs(uint32_t argc)
     if (argc > maxArraySlots)
         return Compile_InlineAbort;
 
+    RootedScript script(cx, script_);
     types::TypeObject *type = types::TypeScript::InitObject(cx, script, PC, JSProto_Array);
     if (!type)
         return Compile_Error;
@@ -875,7 +877,7 @@ mjit::Compiler::inlineNativeFunction(uint32_t argc, bool callingNew)
 
     FrameEntry *origCallee = frame.peek(-((int)argc + 2));
     FrameEntry *thisValue = frame.peek(-((int)argc + 1));
-    types::TypeSet *thisTypes = analysis->poppedTypes(PC, argc);
+    types::StackTypeSet *thisTypes = analysis->poppedTypes(PC, argc);
 
     if (!origCallee->isConstant() || !origCallee->isType(JSVAL_TYPE_OBJECT))
         return Compile_InlineAbort;
@@ -947,7 +949,7 @@ mjit::Compiler::inlineNativeFunction(uint32_t argc, bool callingNew)
         }
     } else if (argc == 1) {
         FrameEntry *arg = frame.peek(-1);
-        types::TypeSet *argTypes = frame.extra(arg).types;
+        types::StackTypeSet *argTypes = frame.extra(arg).types;
         if (!argTypes)
             return Compile_InlineAbort;
         JSValueType argType = arg->isTypeKnown() ? arg->getKnownType() : JSVAL_TYPE_UNKNOWN;
