@@ -195,7 +195,10 @@ RegExpCode::compile(JSContext *cx, JSLinearString &pattern, unsigned *parenCount
             return false;
 
         JSGlobalData globalData(execAlloc);
-        jitCompile(yarrPattern, &globalData, codeBlock);
+        jitCompile(yarrPattern,
+                   JSC::Yarr::Char16,
+                   &globalData,
+                   codeBlock);
         if (!codeBlock.isFallBack())
             return true;
     }
@@ -218,21 +221,23 @@ RegExpRunStatus
 RegExpCode::execute(JSContext *cx, const jschar *chars, size_t length, size_t start,
                     int *output, size_t outputCount)
 {
-    int result;
+    unsigned result;
 #if ENABLE_YARR_JIT
     (void) cx; /* Unused. */
-    if (codeBlock.isFallBack())
-        result = JSC::Yarr::interpret(byteCode, chars, start, length, output);
-    else
-        result = JSC::Yarr::execute(codeBlock, chars, start, length, output);
+    if (codeBlock.isFallBack()) {
+        result = JSC::Yarr::interpret(byteCode, chars, length, start,
+                                      reinterpret_cast<unsigned *>(output));
+    } else {
+        result = codeBlock.execute(chars, start, length, output).start;
+    }
 #else
-    result = JSC::Yarr::interpret(byteCode, chars, start, length, output);
+    result = JSC::Yarr::interpret(byteCode, chars, length, start,
+                                  reinterpret_cast<unsigned *>(output));
 #endif
 
-    if (result == -1)
+    if (result == JSC::Yarr::offsetNoMatch)
         return RegExpRunStatus_Success_NotFound;
 
-    JS_ASSERT(result >= 0);
     return RegExpRunStatus_Success;
 }
 
@@ -519,18 +524,19 @@ RegExpShared::execute(JSContext *cx, const jschar *chars, size_t length, size_t
 /* RegExpCompartment */
 
 RegExpCompartment::RegExpCompartment(JSRuntime *rt)
-  : map_(rt)
+  : map_(rt), inUse_(rt)
 {}
 
 RegExpCompartment::~RegExpCompartment()
 {
-    map_.empty();
+    JS_ASSERT(map_.empty());
+    JS_ASSERT(inUse_.empty());
 }
 
 bool
 RegExpCompartment::init(JSContext *cx)
 {
-    if (!map_.init()) {
+    if (!map_.init() || !inUse_.init()) {
         js_ReportOutOfMemory(cx);
         return false;
     }
@@ -538,12 +544,19 @@ RegExpCompartment::init(JSContext *cx)
     return true;
 }
 
+/* See the comment on RegExpShared lifetime in RegExpObject.h. */
 void
 RegExpCompartment::sweep(JSRuntime *rt)
 {
-    for (Map::Enum e(map_); !e.empty(); e.popFront()) {
-        /* See the comment on RegExpShared lifetime in RegExpObject.h. */
-        RegExpShared *shared = e.front().value;
+#ifdef DEBUG
+    for (Map::Range r = map_.all(); !r.empty(); r.popFront())
+        JS_ASSERT(inUse_.has(r.front().value));
+#endif
+
+    map_.clear();
+
+    for (PendingSet::Enum e(inUse_); !e.empty(); e.popFront()) {
+        RegExpShared *shared = e.front();
         if (shared->activeUseCount == 0 && shared->gcNumberWhenUsed < rt->gcStartNumber) {
             js_delete(shared);
             e.removeFront();
@@ -575,6 +588,12 @@ RegExpCompartment::get(JSContext *cx, JSAtom *keyAtom, JSAtom *source, RegExpFla
         return false;
     }
 
+    if (!inUse_.put(shared)) {
+        map_.remove(key);
+        js_ReportOutOfMemory(cx);
+        return false;
+    }
+
     /*
      * Since 'error' deletes 'shared', only guard 'shared' on success. This is
      * safe since 'shared' cannot be deleted by GC until after the call to
