@@ -17,6 +17,7 @@
 
 #include "jsgcinlines.h"
 #include "jsobjinlines.h"
+#include "jsinterpinlines.h"
 
 #include "jsopcode.h"
 
@@ -42,6 +43,8 @@
 
 using namespace js;
 
+using mozilla::DebugOnly;
+
 /*****************************************************************************/
 
 void
@@ -218,7 +221,7 @@ StackFrame::pcQuadratic(const ContextStack &stack, size_t maxDepth)
 }
 
 bool
-StackFrame::copyRawFrameSlots(CopyVector *vec)
+StackFrame::copyRawFrameSlots(AutoValueVector *vec)
 {
     if (!vec->resize(numFormalArgs() + script()->nfixed))
         return false;
@@ -246,7 +249,7 @@ AssertDynamicScopeMatchesStaticScope(JSScript *script, JSObject *scope)
                 scope = &scope->asClonedBlock().enclosingScope();
                 break;
               case StaticScopeIter::FUNCTION:
-                JS_ASSERT(i.funScript() == scope->asCall().callee().script());
+                JS_ASSERT(scope->asCall().callee().script() == i.funScript());
                 scope = &scope->asCall().enclosingScope();
                 break;
               case StaticScopeIter::NAMED_LAMBDA:
@@ -277,28 +280,30 @@ StackFrame::initCallObject(JSContext *cx)
 bool
 StackFrame::prologue(JSContext *cx, bool newType)
 {
+    RootedScript script(cx, this->script());
+
     JS_ASSERT(!isGeneratorFrame());
-    JS_ASSERT(cx->regs().pc == script()->code);
+    JS_ASSERT(cx->regs().pc == script->code);
 
     if (isEvalFrame()) {
-        if (script()->strictModeCode) {
+        if (script->strictModeCode) {
             CallObject *callobj = CallObject::createForStrictEval(cx, this);
             if (!callobj)
                 return false;
             pushOnScopeChain(*callobj);
             flags_ |= HAS_CALL_OBJ;
         }
-        Probes::enterScript(cx, script(), NULL, this);
+        Probes::enterScript(cx, script, NULL, this);
         return true;
     }
 
     if (isGlobalFrame()) {
-        Probes::enterScript(cx, script(), NULL, this);
+        Probes::enterScript(cx, script, NULL, this);
         return true;
     }
 
     JS_ASSERT(isNonEvalFunctionFrame());
-    AssertDynamicScopeMatchesStaticScope(script(), scopeChain());
+    AssertDynamicScopeMatchesStaticScope(script, scopeChain());
 
     if (fun()->isHeavyweight() && !initCallObject(cx))
         return false;
@@ -311,7 +316,7 @@ StackFrame::prologue(JSContext *cx, bool newType)
         functionThis() = ObjectValue(*obj);
     }
 
-    Probes::enterScript(cx, script(), script()->function(), this);
+    Probes::enterScript(cx, script, script->function(), this);
     return true;
 }
 
@@ -321,7 +326,8 @@ StackFrame::epilogue(JSContext *cx)
     JS_ASSERT(!isYielding());
     JS_ASSERT(!hasBlockChain());
 
-    Probes::exitScript(cx, script(), script()->function(), this);
+    RootedScript script(cx, this->script());
+    Probes::exitScript(cx, script, script->function(), this);
 
     if (isEvalFrame()) {
         if (isStrictEvalFrame()) {
@@ -357,9 +363,9 @@ StackFrame::epilogue(JSContext *cx)
     JS_ASSERT(isNonEvalFunctionFrame());
 
     if (fun()->isHeavyweight())
-        JS_ASSERT_IF(hasCallObj(), scopeChain()->asCall().callee().script() == script());
+        JS_ASSERT_IF(hasCallObj(), scopeChain()->asCall().callee().script() == script);
     else
-        AssertDynamicScopeMatchesStaticScope(script(), scopeChain());
+        AssertDynamicScopeMatchesStaticScope(script, scopeChain());
 
     if (cx->compartment->debugMode())
         cx->runtime->debugScopes->onPopCall(this, cx);
@@ -631,80 +637,14 @@ StackSpace::containingSegment(const StackFrame *target) const
 }
 
 void
-StackSpace::markAndClobberFrame(JSTracer *trc, StackFrame *fp, Value *slotsEnd, jsbytecode *pc)
+StackSpace::markFrame(JSTracer *trc, StackFrame *fp, Value *slotsEnd)
 {
     Value *slotsBegin = fp->slots();
-
-    /* If it's a scripted frame, we should have a pc. */
-    JS_ASSERT(pc);
-
-    JSScript *script = fp->script();
-    if (!script->hasAnalysis() || !script->analysis()->ranLifetimes()) {
-        if (trc)
-            gc::MarkValueRootRange(trc, slotsBegin, slotsEnd, "vm_stack");
-        return;
-    }
-
-    /*
-     * If the JIT ran a lifetime analysis, then it may have left garbage in the
-     * slots considered not live. We need to avoid marking them. Additionally,
-     * in case the analysis information is thrown out later, we overwrite these
-     * dead slots with valid values so that future GCs won't crash. Analysis
-     * results are thrown away during the sweeping phase, so we always have at
-     * least one GC to do this.
-     */
-    JSRuntime *rt = script->compartment()->rt;
-    analyze::AutoEnterAnalysis aea(script->compartment());
-    analyze::ScriptAnalysis *analysis = script->analysis();
-    uint32_t offset = pc - script->code;
-    Value *fixedEnd = slotsBegin + script->nfixed;
-    for (Value *vp = slotsBegin; vp < fixedEnd; vp++) {
-        uint32_t slot = analyze::LocalSlot(script, vp - slotsBegin);
-
-        /* Will this slot be synced by the JIT? */
-        if (!analysis->trackSlot(slot) || analysis->liveness(slot).live(offset)) {
-            if (trc)
-                gc::MarkValueRoot(trc, vp, "vm_stack");
-        } else if (!trc || script->compartment()->isDiscardingJitCode(trc)) {
-            /*
-             * If we're throwing away analysis information, we need to replace
-             * non-live Values with ones that can safely be marked in later
-             * collections.
-             */
-            if (vp->isDouble()) {
-                *vp = DoubleValue(0.0);
-            } else {
-                /*
-                 * It's possible that *vp may not be a valid Value. For example,
-                 * it may be tagged as a NullValue but the low bits may be
-                 * nonzero so that isNull() returns false. This can cause
-                 * problems later on when marking the value. Extracting the type
-                 * in this way and then overwriting the value circumvents the
-                 * problem.
-                 */
-                JSValueType type = vp->extractNonDoubleType();
-                if (type == JSVAL_TYPE_INT32)
-                    *vp = Int32Value(0);
-                else if (type == JSVAL_TYPE_UNDEFINED)
-                    *vp = UndefinedValue();
-                else if (type == JSVAL_TYPE_BOOLEAN)
-                    *vp = BooleanValue(false);
-                else if (type == JSVAL_TYPE_STRING)
-                    *vp = StringValue(rt->atomState.null);
-                else if (type == JSVAL_TYPE_NULL)
-                    *vp = NullValue();
-                else if (type == JSVAL_TYPE_OBJECT)
-                    *vp = ObjectValue(fp->scopeChain()->global());
-            }
-        }
-    }
-
-    if (trc)
-        gc::MarkValueRootRange(trc, fixedEnd, slotsEnd, "vm_stack");
+    gc::MarkValueRootRange(trc, slotsBegin, slotsEnd, "vm_stack");
 }
 
 void
-StackSpace::markAndClobber(JSTracer *trc)
+StackSpace::mark(JSTracer *trc)
 {
     /* NB: this depends on the continuity of segments in memory. */
     Value *nextSegEnd = firstUnused();
@@ -720,21 +660,18 @@ StackSpace::markAndClobber(JSTracer *trc)
          * which gets marked in reverse order.
          */
         Value *slotsEnd = nextSegEnd;
-        jsbytecode *pc = seg->maybepc();
         for (StackFrame *fp = seg->maybefp(); (Value *)fp > (Value *)seg; fp = fp->prev()) {
             /* Mark from fp->slots() to slotsEnd. */
-            markAndClobberFrame(trc, fp, slotsEnd, pc);
+            markFrame(trc, fp, slotsEnd);
 
-            if (trc)
-                fp->mark(trc);
+            fp->mark(trc);
             slotsEnd = (Value *)fp;
 
             InlinedSite *site;
-            pc = fp->prevpc(&site);
+            fp->prevpc(&site);
             JS_ASSERT_IF(fp->prev(), !site);
         }
-        if (trc)
-            gc::MarkValueRootRange(trc, seg->slotsBegin(), slotsEnd, "vm_stack");
+        gc::MarkValueRootRange(trc, seg->slotsBegin(), slotsEnd, "vm_stack");
         nextSegEnd = (Value *)seg;
     }
 }
@@ -751,6 +688,7 @@ StackSpace::markActiveCompartments()
 JS_FRIEND_API(bool)
 StackSpace::ensureSpaceSlow(JSContext *cx, MaybeReportError report, Value *from, ptrdiff_t nvals) const
 {
+    AssertCanGC();
     assertInvariants();
 
     JSCompartment *dest = cx->compartment;
@@ -871,7 +809,13 @@ bool
 StackSpace::containsSlow(StackFrame *fp)
 {
     for (AllFramesIter i(*this); !i.done(); ++i) {
-        if (i.fp() == fp)
+        /*
+         * Debug-mode currently disables Ion compilation in the compartment of
+         * the debuggee.
+         */
+        if (i.isIon())
+            continue;
+        if (i.interpFrame() == fp)
             return true;
     }
     return false;
@@ -911,16 +855,6 @@ ContextStack::onTop() const
     return seg_ && seg_ == space().seg_;
 }
 
-bool
-ContextStack::containsSlow(const StackFrame *target) const
-{
-    for (StackSegment *s = seg_; s; s = s->prevInContext()) {
-        if (s->contains(target))
-            return true;
-    }
-    return false;
-}
-
 /*
  * This helper function brings the ContextStack to the top of the thread stack
  * (so that it can be extended to push a frame and/or arguments) by potentially
@@ -934,6 +868,8 @@ Value *
 ContextStack::ensureOnTop(JSContext *cx, MaybeReportError report, unsigned nvars,
                           MaybeExtend extend, bool *pushedSeg)
 {
+    AssertCanGC();
+
     Value *firstUnused = space().firstUnused();
     FrameRegs *regs = cx->maybeRegs();
 
@@ -1012,6 +948,7 @@ bool
 ContextStack::pushInvokeArgs(JSContext *cx, unsigned argc, InvokeArgsGuard *iag,
                              MaybeReportError report)
 {
+    AssertCanGC();
     JS_ASSERT(argc <= StackSpace::ARGS_LENGTH_MAX);
 
     unsigned nvars = 2 + argc;
@@ -1050,10 +987,11 @@ ContextStack::pushInvokeFrame(JSContext *cx, MaybeReportError report,
                               const CallArgs &args, JSFunction *fun,
                               InitialFrameFlags initial, FrameGuard *fg)
 {
+    AssertCanGC();
     JS_ASSERT(onTop());
     JS_ASSERT(space().firstUnused() == args.end());
 
-    JSScript *script = fun->script();
+    RootedScript script(cx, fun->script());
 
     StackFrame::Flags flags = ToFrameFlags(initial);
     StackFrame *fp = getCallFrame(cx, report, args, fun, script, &flags);
@@ -1085,6 +1023,8 @@ ContextStack::pushExecuteFrame(JSContext *cx, JSScript *script, const Value &thi
                                JSObject &scopeChain, ExecuteType type,
                                StackFrame *evalInFrame, ExecuteFrameGuard *efg)
 {
+    AssertCanGC();
+
     /*
      * Even though global code and indirect eval do not execute in the context
      * of the current frame, prev-link these to the current frame so that the
@@ -1103,8 +1043,13 @@ ContextStack::pushExecuteFrame(JSContext *cx, JSScript *script, const Value &thi
         /* Though the prev-frame is given, need to search for prev-call. */
         StackSegment &seg = cx->stack.space().containingSegment(evalInFrame);
         StackIter iter(cx->runtime, seg);
-        while (!iter.isScript() || iter.fp() != evalInFrame)
+        /* Debug-mode currently disables Ion compilation. */
+        JS_ASSERT(!evalInFrame->runningInIon());
+        JS_ASSERT_IF(evalInFrame->compartment() == iter.compartment(), !iter.isIon());
+        while (!iter.isScript() || iter.isIon() || iter.interpFrame() != evalInFrame) {
             ++iter;
+            JS_ASSERT_IF(evalInFrame->compartment() == iter.compartment(), !iter.isIon());
+        }
         evalInFrameCalls = iter.calls_;
         extend = CANT_EXTEND;
     } else {
@@ -1181,6 +1126,7 @@ ContextStack::popFrame(const FrameGuard &fg)
 bool
 ContextStack::pushGeneratorFrame(JSContext *cx, JSGenerator *gen, GeneratorFrameGuard *gfg)
 {
+    AssertCanGC();
     HeapValue *genvp = gen->stackSnapshot;
     JS_ASSERT(genvp == HeapValueify(gen->fp->generatorArgsSnapshotBegin()));
     unsigned vplen = HeapValueify(gen->fp->generatorArgsSnapshotEnd()) - genvp;
@@ -1250,6 +1196,8 @@ ContextStack::popGeneratorFrame(const GeneratorFrameGuard &gfg)
 bool
 ContextStack::saveFrameChain()
 {
+    AssertCanGC();
+
     bool pushedSeg;
     if (!ensureOnTop(cx_, REPORT_ERROR, 0, CANT_EXTEND, &pushedSeg))
         return false;
@@ -1283,6 +1231,7 @@ StackIter::poisonRegs()
 void
 StackIter::popFrame()
 {
+    AutoAssertNoGC nogc;
     StackFrame *oldfp = fp_;
     JS_ASSERT(seg_->contains(oldfp));
     fp_ = fp_->prev();
@@ -1310,6 +1259,7 @@ StackIter::popCall()
 void
 StackIter::settleOnNewSegment()
 {
+    AutoAssertNoGC nogc;
     if (FrameRegs *regs = seg_->maybeRegs()) {
         pc_ = regs->pc;
         if (fp_)
@@ -1350,6 +1300,8 @@ StackIter::startOnSegment(StackSegment *seg)
 void
 StackIter::settleOnNewState()
 {
+    AutoAssertNoGC nogc;
+
     /*
      * There are elements of the calls_ and fp_ chains that we want to skip
      * over so iterate until we settle on one or until there are no more.
@@ -1378,9 +1330,9 @@ StackIter::settleOnNewState()
                 /* Avoid duplicating logic; seg_ contains fp_, so no iloop. */
                 StackIter tmp = *this;
                 tmp.startOnSegment(seg_);
-                while (!tmp.isScript() || tmp.fp() != fp_)
+                while (!tmp.isScript() || tmp.fp_ != fp_)
                     ++tmp;
-                JS_ASSERT(tmp.state_ == SCRIPTED && tmp.seg_ == seg_ && tmp.fp_ == fp_);
+                JS_ASSERT(tmp.isScript() && tmp.seg_ == seg_ && tmp.fp_ == fp_);
                 *this = tmp;
                 return;
             }
@@ -1451,8 +1403,10 @@ StackIter::settleOnNewState()
 }
 
 StackIter::StackIter(JSContext *cx, SavedOption savedOption)
-  : maybecx_(cx),
-    savedOption_(savedOption)
+  : perThread_(&cx->runtime->mainThread),
+    maybecx_(cx),
+    savedOption_(savedOption),
+    script_(cx, NULL)
 #ifdef JS_ION
     , ionActivations_(cx),
     ionFrames_((uint8_t *)NULL),
@@ -1474,7 +1428,10 @@ StackIter::StackIter(JSContext *cx, SavedOption savedOption)
 }
 
 StackIter::StackIter(JSRuntime *rt, StackSegment &seg)
-  : maybecx_(NULL), savedOption_(STOP_AT_SAVED)
+  : perThread_(&rt->mainThread),
+    maybecx_(NULL),
+    savedOption_(STOP_AT_SAVED),
+    script_(rt, NULL)
 #ifdef JS_ION
     , ionActivations_(rt),
     ionFrames_((uint8_t *)NULL),
@@ -1490,10 +1447,30 @@ StackIter::StackIter(JSRuntime *rt, StackSegment &seg)
     settleOnNewState();
 }
 
+StackIter::StackIter(const StackIter &other)
+  : perThread_(other.perThread_),
+    maybecx_(other.maybecx_),
+    savedOption_(other.savedOption_),
+    state_(other.state_),
+    fp_(other.fp_),
+    calls_(other.calls_),
+    seg_(other.seg_),
+    pc_(other.pc_),
+    script_(perThread_, other.script_),
+    args_(other.args_)
+#ifdef JS_ION
+    , ionActivations_(other.ionActivations_),
+    ionFrames_(other.ionFrames_),
+    ionInlineFrames_(other.ionInlineFrames_)
+#endif
+{
+}
+
 #ifdef JS_ION
 void
 StackIter::popIonFrame()
 {
+    AutoAssertNoGC nogc;
     // Keep fp which describes all ion frames.
     poisonRegs();
     if (ionFrames_.isScripted() && ionInlineFrames_.more()) {
@@ -1553,7 +1530,7 @@ StackIter::operator++()
         settleOnNewState();
         break;
       case ION:
-#ifdef JS_ION      
+#ifdef JS_ION
         popIonFrame();
         break;
 #else
@@ -1569,10 +1546,31 @@ StackIter::operator==(const StackIter &rhs) const
     return done() == rhs.done() &&
            (done() ||
             (isScript() == rhs.isScript() &&
-             ((isScript() && fp() == rhs.fp()) ||
+             ((isScript() && fp_ == rhs.fp_) ||
               (!isScript() && nativeArgs().base() == rhs.nativeArgs().base()))));
 }
 
+JSCompartment *
+StackIter::compartment() const
+{
+    switch (state_) {
+      case DONE:
+        break;
+      case SCRIPTED:
+        return fp_->compartment();
+      case ION:
+#ifdef  JS_ION
+        return ionActivations_.activation()->compartment();
+#else
+        break;
+#endif
+      case NATIVE:
+        return calls_->callee().compartment();
+    }
+    JS_NOT_REACHED("Unexpected state");
+    return NULL;
+}
+
 bool
 StackIter::isFunctionFrame() const
 {
@@ -1580,9 +1578,9 @@ StackIter::isFunctionFrame() const
       case DONE:
         break;
       case SCRIPTED:
-        return fp()->isFunctionFrame();
+        return interpFrame()->isFunctionFrame();
       case ION:
-#ifdef  JS_ION    
+#ifdef  JS_ION
         return ionInlineFrames_.isFunctionFrame();
 #else
         break;
@@ -1601,7 +1599,7 @@ StackIter::isEvalFrame() const
       case DONE:
         break;
       case SCRIPTED:
-        return fp()->isEvalFrame();
+        return interpFrame()->isEvalFrame();
       case ION:
       case NATIVE:
         return false;
@@ -1618,7 +1616,7 @@ StackIter::isNonEvalFunctionFrame() const
       case DONE:
         break;
       case SCRIPTED:
-        return fp()->isNonEvalFunctionFrame();
+        return interpFrame()->isNonEvalFunctionFrame();
       case ION:
       case NATIVE:
         return !isEvalFrame() && isFunctionFrame();
@@ -1632,19 +1630,18 @@ StackIter::isConstructing() const
 {
     switch (state_) {
       case DONE:
-        JS_NOT_REACHED("Unexpected state");
-        return false;
+        break;
       case ION:
-#ifdef JS_ION      
+#ifdef JS_ION
         return ionInlineFrames_.isConstructing();
 #else
-        JS_NOT_REACHED("Unexpected state");
-        return false;
+        break;
 #endif        
       case SCRIPTED:
       case NATIVE:
-        return fp()->isConstructing();
+        return fp_->isConstructing();
     }
+    JS_NOT_REACHED("Unexpected state");
     return false;
 }
 
@@ -1656,16 +1653,16 @@ StackIter::callee() const
         break;
       case SCRIPTED:
         JS_ASSERT(isFunctionFrame());
-        return &fp()->callee();
+        return &interpFrame()->callee();
       case ION:
-#ifdef JS_ION      
+#ifdef JS_ION
         if (ionFrames_.isScripted())
             return ionInlineFrames_.callee();
         JS_ASSERT(ionFrames_.isNative());
         return ionFrames_.callee();
 #else
         break;
-#endif        
+#endif
       case NATIVE:
         return nativeArgs().callee().toFunction();
     }
@@ -1681,7 +1678,7 @@ StackIter::calleev() const
         break;
       case SCRIPTED:
         JS_ASSERT(isFunctionFrame());
-        return fp()->calleev();
+        return interpFrame()->calleev();
       case ION:
 #ifdef JS_ION
         return ObjectValue(*callee());
@@ -1703,7 +1700,7 @@ StackIter::numActualArgs() const
         break;
       case SCRIPTED:
         JS_ASSERT(isFunctionFrame());
-        return fp()->numActualArgs();
+        return interpFrame()->numActualArgs();
       case ION:
 #ifdef JS_ION
         return ionInlineFrames_.numActualArgs();
@@ -1717,6 +1714,37 @@ StackIter::numActualArgs() const
     return 0;
 }
 
+JSObject *
+StackIter::scopeChain() const
+{
+    switch (state_) {
+      case DONE:
+        break;
+      case ION:
+#ifdef JS_ION
+        return ionInlineFrames_.scopeChain();
+#else
+        break;
+#endif
+      case SCRIPTED:
+        return interpFrame()->scopeChain();
+      case NATIVE:
+        break;
+    }
+    JS_NOT_REACHED("Unexpected state");
+    return NULL;
+}
+
+bool
+StackIter::computeThis() const
+{
+    if (isScript() && !isIon()) {
+        JS_ASSERT(maybecx_);
+        return ComputeThis(maybecx_, interpFrame());
+    }
+    return true;
+}
+
 Value
 StackIter::thisv() const
 {
@@ -1731,7 +1759,7 @@ StackIter::thisv() const
 #endif
       case SCRIPTED:
       case NATIVE:
-        return fp()->thisValue();
+        return fp_->thisValue();
     }
     JS_NOT_REACHED("Unexpected state");
     return NullValue();
@@ -1740,6 +1768,7 @@ StackIter::thisv() const
 size_t
 StackIter::numFrameSlots() const
 {
+    AutoAssertNoGC nogc;
     switch (state_) {
       case DONE:
       case NATIVE:
@@ -1752,8 +1781,8 @@ StackIter::numFrameSlots() const
 #endif
       case SCRIPTED:
         JS_ASSERT(maybecx_);
-        JS_ASSERT(maybecx_->regs().spForStackDepth(0) == fp()->base());
-        return maybecx_->regs().sp - fp()->base();
+        JS_ASSERT(maybecx_->regs().spForStackDepth(0) == interpFrame()->base());
+        return maybecx_->regs().sp - interpFrame()->base();
     }
     JS_NOT_REACHED("Unexpected state");
     return 0;
@@ -1762,6 +1791,7 @@ StackIter::numFrameSlots() const
 Value
 StackIter::frameSlotValue(size_t index) const
 {
+    AutoAssertNoGC nogc;
     switch (state_) {
       case DONE:
       case NATIVE:
@@ -1777,7 +1807,7 @@ StackIter::frameSlotValue(size_t index) const
         break;
 #endif
       case SCRIPTED:
-          return fp()->base()[index];
+          return interpFrame()->base()[index];
     }
     JS_NOT_REACHED("Unexpected state");
     return NullValue();
