@@ -11,6 +11,7 @@
 #include "nsHTMLParts.h"
 #include "nsContainerFrame.h"
 #include "nsGenericHTMLElement.h"
+#include "nsAttrValueInlines.h"
 #include "nsPresContext.h"
 #include "nsIPresShell.h"
 #include "nsIDocument.h"
@@ -33,7 +34,7 @@
 
 using namespace mozilla;
 
-NS_DECLARE_FRAME_PROPERTY(FontSizeInflationProperty, nsnull)
+NS_DECLARE_FRAME_PROPERTY(FontSizeInflationProperty, nullptr)
 
 NS_IMPL_FRAMEARENA_HELPERS(nsBulletFrame)
 
@@ -51,11 +52,11 @@ nsBulletFrame::DestroyFrom(nsIFrame* aDestructRoot)
                                           mImageRequest,
                                           &mRequestRegistered);
     mImageRequest->CancelAndForgetObserver(NS_ERROR_FAILURE);
-    mImageRequest = nsnull;
+    mImageRequest = nullptr;
   }
 
   if (mListener) {
-    mListener->SetFrame(nsnull);
+    mListener->SetFrame(nullptr);
   }
 
   // Let base class do the rest
@@ -118,8 +119,8 @@ nsBulletFrame::DidSetStyleContext(nsStyleContext* aOldStyleContext)
         } else {
           nsLayoutUtils::DeregisterImageRequest(PresContext(), mImageRequest,
                                                 &mRequestRegistered);
-          mImageRequest->Cancel(NS_ERROR_FAILURE);
-          mImageRequest = nsnull;
+          mImageRequest->CancelAndForgetObserver(NS_ERROR_FAILURE);
+          mImageRequest = nullptr;
         }
       }
     }
@@ -138,8 +139,8 @@ nsBulletFrame::DidSetStyleContext(nsStyleContext* aOldStyleContext)
       nsLayoutUtils::DeregisterImageRequest(PresContext(), mImageRequest,
                                             &mRequestRegistered);
 
-      mImageRequest->Cancel(NS_ERROR_FAILURE);
-      mImageRequest = nsnull;
+      mImageRequest->CancelAndForgetObserver(NS_ERROR_FAILURE);
+      mImageRequest = nullptr;
     }
   }
 
@@ -226,10 +227,10 @@ nsBulletFrame::PaintBullet(nsRenderingContext& aRenderingContext, nsPoint aPt,
                            const nsRect& aDirtyRect)
 {
   const nsStyleList* myList = GetStyleList();
-  PRUint8 listStyleType = myList->mListStyleType;
+  uint8_t listStyleType = myList->mListStyleType;
 
   if (myList->GetListStyleImage() && mImageRequest) {
-    PRUint32 status;
+    uint32_t status;
     mImageRequest->GetImageStatus(&status);
     if (status & imgIRequest::STATUS_LOAD_COMPLETE &&
         !(status & imgIRequest::STATUS_ERROR)) {
@@ -350,12 +351,16 @@ nsBulletFrame::PaintBullet(nsRenderingContext& aRenderingContext, nsPoint aPt,
   }
 }
 
-PRInt32
-nsBulletFrame::SetListItemOrdinal(PRInt32 aNextOrdinal,
-                                  bool* aChanged)
+int32_t
+nsBulletFrame::SetListItemOrdinal(int32_t aNextOrdinal,
+                                  bool* aChanged,
+                                  int32_t aIncrement)
 {
+  MOZ_ASSERT(aIncrement == 1 || aIncrement == -1,
+             "We shouldn't have weird increments here");
+
   // Assume that the ordinal comes from the caller
-  PRInt32 oldOrdinal = mOrdinal;
+  int32_t oldOrdinal = mOrdinal;
   mOrdinal = aNextOrdinal;
 
   // Try to get value directly from the list-item, if it specifies a
@@ -376,7 +381,7 @@ nsBulletFrame::SetListItemOrdinal(PRInt32 aNextOrdinal,
 
   *aChanged = oldOrdinal != mOrdinal;
 
-  return mOrdinal + 1;
+  return mOrdinal + aIncrement;
 }
 
 
@@ -388,21 +393,21 @@ nsBulletFrame::SetListItemOrdinal(PRInt32 aNextOrdinal,
  * could represent mOrder in the desired numbering system.  false
  * means we had to fall back to decimal
  */
-static bool DecimalToText(PRInt32 ordinal, nsString& result)
+static bool DecimalToText(int32_t ordinal, nsString& result)
 {
    char cbuf[40];
    PR_snprintf(cbuf, sizeof(cbuf), "%ld", ordinal);
    result.AppendASCII(cbuf);
    return true;
 }
-static bool DecimalLeadingZeroToText(PRInt32 ordinal, nsString& result)
+static bool DecimalLeadingZeroToText(int32_t ordinal, nsString& result)
 {
    char cbuf[40];
    PR_snprintf(cbuf, sizeof(cbuf), "%02ld", ordinal);
    result.AppendASCII(cbuf);
    return true;
 }
-static bool OtherDecimalToText(PRInt32 ordinal, PRUnichar zeroChar, nsString& result)
+static bool OtherDecimalToText(int32_t ordinal, PRUnichar zeroChar, nsString& result)
 {
    PRUnichar diff = zeroChar - PRUnichar('0');
    DecimalToText(ordinal, result);
@@ -411,11 +416,11 @@ static bool OtherDecimalToText(PRInt32 ordinal, PRUnichar zeroChar, nsString& re
      // skip the leading '-'
      ++p;
    }     
-   for(; nsnull != *p ; p++) 
+   for(; '\0' != *p ; p++) 
       *p += diff;
    return true;
 }
-static bool TamilToText(PRInt32 ordinal,  nsString& result)
+static bool TamilToText(int32_t ordinal,  nsString& result)
 {
    PRUnichar diff = 0x0BE6 - PRUnichar('0');
    DecimalToText(ordinal, result); 
@@ -424,7 +429,7 @@ static bool TamilToText(PRInt32 ordinal,  nsString& result)
      return false;
    }
    PRUnichar* p = result.BeginWriting();
-   for(; nsnull != *p ; p++) 
+   for(; '\0' != *p ; p++) 
       if(*p != PRUnichar('0'))
          *p += diff;
    return true;
@@ -436,7 +441,7 @@ static const char gUpperRomanCharsA[] = "IXCM";
 static const char gLowerRomanCharsB[] = "vld";
 static const char gUpperRomanCharsB[] = "VLD";
 
-static bool RomanToText(PRInt32 ordinal, nsString& result, const char* achars, const char* bchars)
+static bool RomanToText(int32_t ordinal, nsString& result, const char* achars, const char* bchars)
 {
   if (ordinal < 1 || ordinal > 3999) {
     DecimalToText(ordinal, result);
@@ -444,11 +449,11 @@ static bool RomanToText(PRInt32 ordinal, nsString& result, const char* achars, c
   }
   nsAutoString addOn, decStr;
   decStr.AppendInt(ordinal, 10);
-  PRIntn len = decStr.Length();
+  int len = decStr.Length();
   const PRUnichar* dp = decStr.get();
   const PRUnichar* end = dp + len;
-  PRIntn romanPos = len;
-  PRIntn n;
+  int romanPos = len;
+  int n;
 
   for (; dp < end; dp++) {
     romanPos--;
@@ -663,17 +668,17 @@ static const PRUnichar gEthiopicHalehameTiEtChars[ETHIOPIC_HALEHAME_TI_ET_CHARS_
 
 #define NUM_BUF_SIZE 34 
 
-static bool CharListToText(PRInt32 ordinal, nsString& result, const PRUnichar* chars, PRInt32 aBase)
+static bool CharListToText(int32_t ordinal, nsString& result, const PRUnichar* chars, int32_t aBase)
 {
   PRUnichar buf[NUM_BUF_SIZE];
-  PRInt32 idx = NUM_BUF_SIZE;
+  int32_t idx = NUM_BUF_SIZE;
   if (ordinal < 1) {
     DecimalToText(ordinal, result);
     return false;
   }
   do {
     ordinal--; // a == 0
-    PRInt32 cur = ordinal % aBase;
+    int32_t cur = ordinal % aBase;
     buf[--idx] = chars[cur];
     ordinal /= aBase ;
   } while ( ordinal > 0);
@@ -718,13 +723,13 @@ static const PRUnichar gCJKIdeographic10KUnit3[4] =
   0x000, 0x4E07, 0x5104, 0x5146
 };
 
-static const bool CJKIdeographicToText(PRInt32 ordinal, nsString& result, 
+static const bool CJKIdeographicToText(int32_t ordinal, nsString& result, 
                                    const PRUnichar* digits,
                                    const PRUnichar *unit, 
                                    const PRUnichar* unit10k)
 {
 // In theory, we need the following if condiction,
-// However, the limit, 10 ^ 16, is greater than the max of PRUint32
+// However, the limit, 10 ^ 16, is greater than the max of uint32_t
 // so we don't really need to test it here.
 // if( ordinal > 9999999999999999)
 // {
@@ -740,15 +745,15 @@ static const bool CJKIdeographicToText(PRInt32 ordinal, nsString& result,
   PRUnichar c10kUnit = 0;
   PRUnichar cUnit = 0;
   PRUnichar cDigit = 0;
-  PRUint32 ud = 0;
+  uint32_t ud = 0;
   PRUnichar buf[NUM_BUF_SIZE];
-  PRInt32 idx = NUM_BUF_SIZE;
+  int32_t idx = NUM_BUF_SIZE;
   bool bOutputZero = ( 0 == ordinal );
   do {
     if(0 == (ud % 4)) {
       c10kUnit = unit10k[ud/4];
     }
-    PRInt32 cur = ordinal % 10;
+    int32_t cur = ordinal % 10;
     cDigit = digits[cur];
     if( 0 == cur)
     {
@@ -794,7 +799,7 @@ static const PRUnichar gHebrewDigit[22] =
 0x05E7, 0x05E8, 0x05E9, 0x05EA
 };
 
-static bool HebrewToText(PRInt32 ordinal, nsString& result)
+static bool HebrewToText(int32_t ordinal, nsString& result)
 {
   if (ordinal < 1 || ordinal > 999999) {
     DecimalToText(ordinal, result);
@@ -804,9 +809,9 @@ static bool HebrewToText(PRInt32 ordinal, nsString& result)
   nsAutoString allText, thousandsGroup;
   do {
     thousandsGroup.Truncate();
-    PRInt32 n3 = ordinal % 1000;
+    int32_t n3 = ordinal % 1000;
     // Process digit for 100 - 900
-    for(PRInt32 n1 = 400; n1 > 0; )
+    for(int32_t n1 = 400; n1 > 0; )
     {
       if( n3 >= n1)
       {
@@ -818,7 +823,7 @@ static bool HebrewToText(PRInt32 ordinal, nsString& result)
     } // for
 
     // Process digit for 10 - 90
-    PRInt32 n2;
+    int32_t n2;
     if( n3 >= 10 )
     {
       // Special process for 15 and 16
@@ -853,7 +858,7 @@ static bool HebrewToText(PRInt32 ordinal, nsString& result)
 }
 
 
-static bool ArmenianToText(PRInt32 ordinal, nsString& result)
+static bool ArmenianToText(int32_t ordinal, nsString& result)
 {
   if (ordinal < 1 || ordinal > 9999) { // zero or reach the limit of Armenian numbering system
     DecimalToText(ordinal, result);
@@ -861,10 +866,10 @@ static bool ArmenianToText(PRInt32 ordinal, nsString& result)
   }
 
   PRUnichar buf[NUM_BUF_SIZE];
-  PRInt32 idx = NUM_BUF_SIZE;
-  PRInt32 d = 0;
+  int32_t idx = NUM_BUF_SIZE;
+  int32_t d = 0;
   do {
-    PRInt32 cur = ordinal % 10;
+    int32_t cur = ordinal % 10;
     if (cur > 0)
     {
       PRUnichar u = 0x0530 + (d * 9) + cur;
@@ -890,7 +895,7 @@ static const PRUnichar gGeorgianValue [ 37 ] = { // 4 * 9 + 1 = 37
 //  10000
    0x10F5
 };
-static bool GeorgianToText(PRInt32 ordinal, nsString& result)
+static bool GeorgianToText(int32_t ordinal, nsString& result)
 {
   if (ordinal < 1 || ordinal > 19999) { // zero or reach the limit of Georgian numbering system
     DecimalToText(ordinal, result);
@@ -898,10 +903,10 @@ static bool GeorgianToText(PRInt32 ordinal, nsString& result)
   }
 
   PRUnichar buf[NUM_BUF_SIZE];
-  PRInt32 idx = NUM_BUF_SIZE;
-  PRInt32 d = 0;
+  int32_t idx = NUM_BUF_SIZE;
+  int32_t d = 0;
   do {
-    PRInt32 cur = ordinal % 10;
+    int32_t cur = ordinal % 10;
     if (cur > 0)
     {
       PRUnichar u = gGeorgianValue[(d * 9 ) + ( cur - 1)];
@@ -924,7 +929,7 @@ static bool GeorgianToText(PRInt32 ordinal, nsString& result)
 #define ETHIOPIC_HUNDRED         0x137B
 #define ETHIOPIC_TEN_THOUSAND    0x137C
 
-static bool EthiopicToText(PRInt32 ordinal, nsString& result)
+static bool EthiopicToText(int32_t ordinal, nsString& result)
 {
   nsAutoString asciiNumberString;      // decimal string representation of ordinal
   DecimalToText(ordinal, asciiNumberString);
@@ -932,7 +937,7 @@ static bool EthiopicToText(PRInt32 ordinal, nsString& result)
     result.Append(asciiNumberString);
     return false;
   }
-  PRUint8 asciiStringLength = asciiNumberString.Length();
+  uint8_t asciiStringLength = asciiNumberString.Length();
 
   // If number length is odd, add a leading "0"
   // the leading "0" preconditions the string to always have the
@@ -948,12 +953,12 @@ static bool EthiopicToText(PRInt32 ordinal, nsString& result)
   // Iterate from the highest digits to lowest
   // indexFromLeft       indexes digits (0 = most significant)
   // groupIndexFromRight indexes pairs of digits (0 = least significant)
-  for (PRUint8 indexFromLeft = 0, groupIndexFromRight = asciiStringLength >> 1;
+  for (uint8_t indexFromLeft = 0, groupIndexFromRight = asciiStringLength >> 1;
        indexFromLeft <= asciiStringLength;
        indexFromLeft += 2, groupIndexFromRight--) {
-    PRUint8 tensValue  = asciiNumberString.CharAt(indexFromLeft) & 0x0F;
-    PRUint8 unitsValue = asciiNumberString.CharAt(indexFromLeft + 1) & 0x0F;
-    PRUint8 groupValue = tensValue * 10 + unitsValue;
+    uint8_t tensValue  = asciiNumberString.CharAt(indexFromLeft) & 0x0F;
+    uint8_t unitsValue = asciiNumberString.CharAt(indexFromLeft + 1) & 0x0F;
+    uint8_t groupValue = tensValue * 10 + unitsValue;
 
     bool oddGroup = (groupIndexFromRight & 1);
 
@@ -990,8 +995,8 @@ static bool EthiopicToText(PRInt32 ordinal, nsString& result)
 
 
 /* static */ bool
-nsBulletFrame::AppendCounterText(PRInt32 aListStyleType,
-                                 PRInt32 aOrdinal,
+nsBulletFrame::AppendCounterText(int32_t aListStyleType,
+                                 int32_t aOrdinal,
                                  nsString& result)
 {
   bool success = true;
@@ -1271,7 +1276,7 @@ nsBulletFrame::GetDesiredSize(nsPresContext*  aCX,
   RemoveStateBits(BULLET_FRAME_IMAGE_LOADING);
 
   if (myList->GetListStyleImage() && mImageRequest) {
-    PRUint32 status;
+    uint32_t status;
     mImageRequest->GetImageStatus(&status);
     if (status & imgIRequest::STATUS_SIZE_AVAILABLE &&
         !(status & imgIRequest::STATUS_ERROR)) {
@@ -1426,14 +1431,41 @@ nsBulletFrame::GetPrefWidth(nsRenderingContext *aRenderingContext)
   return metrics.width;
 }
 
+NS_IMETHODIMP
+nsBulletFrame::Notify(imgIRequest *aRequest, int32_t aType, const nsIntRect* aData)
+{
+  if (aType == imgINotificationObserver::SIZE_AVAILABLE) {
+    nsCOMPtr<imgIContainer> image;
+    aRequest->GetImage(getter_AddRefs(image));
+    return OnStartContainer(aRequest, image);
+  }
 
-NS_IMETHODIMP nsBulletFrame::OnStartContainer(imgIRequest *aRequest,
-                                              imgIContainer *aImage)
+  if (aType == imgINotificationObserver::FRAME_UPDATE) {
+    // The image has changed.
+    // Invalidate the entire content area. Maybe it's not optimal but it's simple and
+    // always correct, and I'll be a stunned mullet if it ever matters for performance
+    InvalidateFrame();
+  }
+
+  if (aType == imgINotificationObserver::IS_ANIMATED) {
+    // Register the image request with the refresh driver now that we know it's
+    // animated.
+    if (aRequest == mImageRequest) {
+      nsLayoutUtils::RegisterImageRequest(PresContext(), mImageRequest,
+                                          &mRequestRegistered);
+    }
+  }
+
+  return NS_OK;
+}
+
+nsresult nsBulletFrame::OnStartContainer(imgIRequest *aRequest,
+                                         imgIContainer *aImage)
 {
   if (!aImage) return NS_ERROR_INVALID_ARG;
   if (!aRequest) return NS_ERROR_INVALID_ARG;
 
-  PRUint32 status;
+  uint32_t status;
   aRequest->GetImageStatus(&status);
   if (status & imgIRequest::STATUS_ERROR) {
     return NS_OK;
@@ -1470,67 +1502,13 @@ NS_IMETHODIMP nsBulletFrame::OnStartContainer(imgIRequest *aRequest,
   return NS_OK;
 }
 
-NS_IMETHODIMP nsBulletFrame::OnDataAvailable(imgIRequest *aRequest,
-                                             bool aCurrentFrame,
-                                             const nsIntRect *aRect)
-{
-  // The image has changed.
-  // Invalidate the entire content area. Maybe it's not optimal but it's simple and
-  // always correct, and I'll be a stunned mullet if it ever matters for performance
-  Invalidate(nsRect(0, 0, mRect.width, mRect.height));
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsBulletFrame::OnStopDecode(imgIRequest *aRequest,
-                                          nsresult aStatus,
-                                          const PRUnichar *aStatusArg)
-{
-  // XXX should the bulletframe do anything if the image failed to load?
-  //     it didn't in the old code...
-
-#if 0
-  if (NS_FAILED(aStatus)) {
-    // We failed to load the image. Notify the pres shell
-    if (NS_FAILED(aStatus) && (mImageRequest == aRequest || !mImageRequest)) {
-      imageFailed = true;
-    }
-  }
-#endif
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsBulletFrame::OnImageIsAnimated(imgIRequest* aRequest)
-{
-  // Register the image request with the refresh driver now that we know it's
-  // animated.
-  if (aRequest == mImageRequest) {
-    nsLayoutUtils::RegisterImageRequest(PresContext(), mImageRequest,
-                                        &mRequestRegistered);
-  }
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsBulletFrame::FrameChanged(imgIRequest *aRequest,
-                                          imgIContainer *aContainer,
-                                          const nsIntRect *aDirtyRect)
-{
-  // Invalidate the entire content area. Maybe it's not optimal but it's simple and
-  // always correct.
-  Invalidate(nsRect(0, 0, mRect.width, mRect.height));
-
-  return NS_OK;
-}
-
 void
 nsBulletFrame::GetLoadGroup(nsPresContext *aPresContext, nsILoadGroup **aLoadGroup)
 {
   if (!aPresContext)
     return;
 
-  NS_PRECONDITION(nsnull != aLoadGroup, "null OUT parameter pointer");
+  NS_PRECONDITION(nullptr != aLoadGroup, "null OUT parameter pointer");
 
   nsIPresShell *shell = aPresContext->GetPresShell();
 
@@ -1545,7 +1523,7 @@ nsBulletFrame::GetLoadGroup(nsPresContext *aPresContext, nsILoadGroup **aLoadGro
 }
 
 union VoidPtrOrFloat {
-  VoidPtrOrFloat() : p(nsnull) {}
+  VoidPtrOrFloat() : p(nullptr) {}
 
   void *p;
   float f;
@@ -1619,10 +1597,10 @@ nsBulletFrame::GetBaseline() const
 
 
 
-NS_IMPL_ISUPPORTS2(nsBulletListener, imgIDecoderObserver, imgIContainerObserver)
+NS_IMPL_ISUPPORTS1(nsBulletListener, imgINotificationObserver)
 
 nsBulletListener::nsBulletListener() :
-  mFrame(nsnull)
+  mFrame(nullptr)
 {
 }
 
@@ -1630,49 +1608,10 @@ nsBulletListener::~nsBulletListener()
 {
 }
 
-NS_IMETHODIMP nsBulletListener::OnStartContainer(imgIRequest *aRequest,
-                                                 imgIContainer *aImage)
+NS_IMETHODIMP
+nsBulletListener::Notify(imgIRequest *aRequest, int32_t aType, const nsIntRect* aData)
 {
   if (!mFrame)
     return NS_ERROR_FAILURE;
-
-  return mFrame->OnStartContainer(aRequest, aImage);
-}
-
-NS_IMETHODIMP nsBulletListener::OnDataAvailable(imgIRequest *aRequest,
-                                                bool aCurrentFrame,
-                                                const nsIntRect *aRect)
-{
-  if (!mFrame)
-    return NS_OK;
-
-  return mFrame->OnDataAvailable(aRequest, aCurrentFrame, aRect);
-}
-
-NS_IMETHODIMP nsBulletListener::OnStopDecode(imgIRequest *aRequest,
-                                             nsresult status,
-                                             const PRUnichar *statusArg)
-{
-  if (!mFrame)
-    return NS_OK;
-  
-  return mFrame->OnStopDecode(aRequest, status, statusArg);
-}
-
-NS_IMETHODIMP nsBulletListener::OnImageIsAnimated(imgIRequest *aRequest)
-{
-  if (!mFrame)
-    return NS_OK;
-
-  return mFrame->OnImageIsAnimated(aRequest);
-}
-
-NS_IMETHODIMP nsBulletListener::FrameChanged(imgIRequest *aRequest,
-                                             imgIContainer *aContainer,
-                                             const nsIntRect *aDirtyRect)
-{
-  if (!mFrame)
-    return NS_OK;
-
-  return mFrame->FrameChanged(aRequest, aContainer, aDirtyRect);
+  return mFrame->Notify(aRequest, aType, aData);
 }
