@@ -5,12 +5,16 @@
 
 package org.mozilla.gecko;
 
+import org.mozilla.gecko.db.BrowserDB;
+import org.mozilla.gecko.db.BrowserContract.Combined;
+import org.mozilla.gecko.util.GeckoAsyncTask;
+
 import android.app.Activity;
 import android.app.AlertDialog;
-import android.content.DialogInterface;
-import android.content.Intent;
 import android.content.ContentResolver;
 import android.content.Context;
+import android.content.DialogInterface;
+import android.content.Intent;
 import android.content.res.Configuration;
 import android.graphics.Bitmap;
 import android.graphics.BitmapFactory;
@@ -23,17 +27,15 @@ import android.text.InputType;
 import android.text.Spanned;
 import android.text.TextUtils;
 import android.text.TextWatcher;
-import android.util.AttributeSet;
 import android.util.Log;
-import android.view.ActionMode;
 import android.view.ContextMenu;
 import android.view.ContextMenu.ContextMenuInfo;
 import android.view.KeyEvent;
 import android.view.LayoutInflater;
 import android.view.MenuItem;
 import android.view.View;
-import android.view.inputmethod.InputMethodManager;
 import android.view.inputmethod.EditorInfo;
+import android.view.inputmethod.InputMethodManager;
 import android.widget.Button;
 import android.widget.EditText;
 import android.widget.ImageButton;
@@ -41,9 +43,6 @@ import android.widget.ListView;
 import android.widget.TabWidget;
 import android.widget.Toast;
 
-import org.mozilla.gecko.db.BrowserDB;
-import org.mozilla.gecko.db.BrowserContract.Bookmarks;
-
 import java.net.URLEncoder;
 import java.util.Arrays;
 import java.util.Collection;
@@ -328,12 +327,15 @@ public class AwesomeBar extends GeckoActivity {
         mGoButton.setVisibility(View.VISIBLE);
 
         int imageResource = R.drawable.ic_awesomebar_go;
+        String contentDescription = getString(R.string.go);
         int imeAction = EditorInfo.IME_ACTION_GO;
         if (isSearchUrl(text)) {
             imageResource = R.drawable.ic_awesomebar_search;
+            contentDescription = getString(R.string.search);
             imeAction = EditorInfo.IME_ACTION_SEARCH;
         }
         mGoButton.setImageResource(imageResource);
+        mGoButton.setContentDescription(contentDescription);
 
         int actionBits = mText.getImeOptions() & EditorInfo.IME_MASK_ACTION;
         if (actionBits != imeAction) {
@@ -365,12 +367,20 @@ public class AwesomeBar extends GeckoActivity {
 
     private void openUserEnteredAndFinish(String url) {
         int index = url.indexOf(' ');
-        if (index != -1) {
-            String keywordUrl = BrowserDB.getUrlForKeyword(mResolver, url.substring(0, index));
-            if (keywordUrl != null && keywordUrl.contains("%s")) {
-                String search = URLEncoder.encode(url.substring(index + 1));
-                url = keywordUrl.replace("%s", search);
-            }
+        String keywordUrl = null;
+        String keywordSearch = null;
+
+        if (index == -1) {
+            keywordUrl = BrowserDB.getUrlForKeyword(mResolver, url);
+            keywordSearch = "";
+        } else {
+            keywordUrl = BrowserDB.getUrlForKeyword(mResolver, url.substring(0, index));
+            keywordSearch = url.substring(index + 1);
+        }
+
+        if (keywordUrl != null) {
+            String search = URLEncoder.encode(keywordSearch);
+            url = keywordUrl.replace("%s", search);
         }
 
         Intent resultIntent = new Intent();
@@ -415,28 +425,20 @@ public class AwesomeBar extends GeckoActivity {
              imm.showSoftInput(mText, InputMethodManager.SHOW_IMPLICIT);
              return true;
         } else {
-            int selStart = -1;
-            int selEnd = -1;
-            if (mText.hasSelection()) {
-                selStart = mText.getSelectionStart();
-                selEnd = mText.getSelectionEnd();
-            }
+            int prevSelStart = mText.getSelectionStart();
+            int prevSelEnd = mText.getSelectionEnd();
 
-            if (selStart >= 0) {
-                // Restore the selection, which gets lost due to the focus switch
-                mText.setSelection(selStart, selEnd);
-            }
-
-            // Manually dispatch the key event to the AwesomeBar before restoring (default) input
-            // focus. dispatchKeyEvent() will update AwesomeBar's cursor position.
+            // Manually dispatch the key event to the AwesomeBar. If selection changed as
+            // a result of the key event, then give focus back to mText
             mText.dispatchKeyEvent(event);
-            int newCursorPos = mText.getSelectionEnd();
-
-            // requestFocusFromTouch() will select all AwesomeBar text, so we must restore cursor
-            // position so subsequent typing does not overwrite all text.
-            mText.requestFocusFromTouch();
-            mText.setSelection(newCursorPos);
 
+            int curSelStart = mText.getSelectionStart();
+            int curSelEnd = mText.getSelectionEnd();
+            if (prevSelStart != curSelStart || prevSelEnd != curSelEnd) {
+                mText.requestFocusFromTouch();
+                // Restore the selection, which gets lost due to the focus switch
+                mText.setSelection(curSelStart, curSelEnd);
+            }
             return true;
         }
     }
@@ -475,13 +477,19 @@ public class AwesomeBar extends GeckoActivity {
         public byte[] favicon;
         public String title;
         public String keyword;
+        public int display;
 
         public ContextMenuSubject(int id, String url, byte[] favicon, String title, String keyword) {
+            this(id, url, favicon, title, keyword, Combined.DISPLAY_NORMAL);
+        }
+
+        public ContextMenuSubject(int id, String url, byte[] favicon, String title, String keyword, int display) {
             this.id = id;
             this.url = url;
             this.favicon = favicon;
             this.title = title;
             this.keyword = keyword;
+            this.display = display;
         }
     };
 
@@ -503,6 +511,7 @@ public class AwesomeBar extends GeckoActivity {
         final byte[] b = mContextMenuSubject.favicon;
         final String title = mContextMenuSubject.title;
         final String keyword = mContextMenuSubject.keyword;
+        final int display = mContextMenuSubject.display;
 
         switch (item.getItemId()) {
             case R.id.open_new_tab: {
@@ -511,10 +520,23 @@ public class AwesomeBar extends GeckoActivity {
                     break;
                 }
 
-                GeckoApp.mAppContext.loadUrl(url, AwesomeBar.Target.NEW_TAB);
+                String newTabUrl = url;
+                if (display == Combined.DISPLAY_READER)
+                    newTabUrl = ReaderModeUtils.getAboutReaderForUrl(url, true);
+
+                Tabs.getInstance().loadUrl(newTabUrl, Tabs.LOADURL_NEW_TAB);
                 Toast.makeText(this, R.string.new_tab_opened, Toast.LENGTH_SHORT).show();
                 break;
             }
+            case R.id.open_in_reader: {
+                if (url == null) {
+                    Log.e(LOGTAG, "Can't open in reader mode because URL is null");
+                    break;
+                }
+
+                openUrlAndFinish(ReaderModeUtils.getAboutReaderForUrl(url, true));
+                break;
+            }
             case R.id.edit_bookmark: {
                 AlertDialog.Builder editPrompt = new AlertDialog.Builder(this);
                 View editView = getLayoutInflater().inflate(R.layout.bookmark_edit, null);
@@ -530,7 +552,7 @@ public class AwesomeBar extends GeckoActivity {
 
                 editPrompt.setPositiveButton(R.string.button_ok, new DialogInterface.OnClickListener() {
                     public void onClick(DialogInterface dialog, int whichButton) {
-                        (new GeckoAsyncTask<Void, Void, Void>() {
+                        (new GeckoAsyncTask<Void, Void, Void>(GeckoApp.mAppContext, GeckoAppShell.getHandler()) {
                             @Override
                             public Void doInBackground(Void... params) {
                                 String newUrl = locationText.getText().toString().trim();
@@ -606,7 +628,7 @@ public class AwesomeBar extends GeckoActivity {
                 break;
             }
             case R.id.remove_history: {
-                (new GeckoAsyncTask<Void, Void, Void>() {
+                (new GeckoAsyncTask<Void, Void, Void>(GeckoApp.mAppContext, GeckoAppShell.getHandler()) {
                     @Override
                     public Void doInBackground(Void... params) {
                         BrowserDB.removeHistoryEntry(mResolver, id);
