@@ -46,6 +46,7 @@
 
 #include "nsThreadUtils.h"
 #include "mozilla/Telemetry.h"
+#include "mozilla/Util.h"
 
 static const char DISK_CACHE_DEVICE_ID[] = { "disk" };
 using namespace mozilla;
@@ -536,7 +537,6 @@ nsDiskCacheDevice::FindEntry(nsCString * key, bool *collision)
 nsresult
 nsDiskCacheDevice::DeactivateEntry(nsCacheEntry * entry)
 {
-    nsresult              rv = NS_OK;
     nsDiskCacheBinding * binding = GetCacheEntryBinding(entry);
     if (!IsValidBinding(binding))
         return NS_ERROR_UNEXPECTED;
@@ -550,7 +550,7 @@ nsDiskCacheDevice::DeactivateEntry(nsCacheEntry * entry)
     // ensure we can cancel the event via the binding later if necessary
     binding->mDeactivateEvent = event;
 
-    rv = nsCacheService::DispatchToCacheIOThread(event);
+    DebugOnly<nsresult> rv = nsCacheService::DispatchToCacheIOThread(event);
     NS_ASSERTION(NS_SUCCEEDED(rv), "DeactivateEntry: Failed dispatching "
                                    "deactivation event");
     return NS_OK;
@@ -1160,3 +1160,15 @@ nsDiskCacheDevice::SetMaxEntrySize(int32_t maxSizeInKilobytes)
     else
         mMaxEntrySize = -1;
 }
+
+size_t
+nsDiskCacheDevice::SizeOfIncludingThis(nsMallocSizeOfFun aMallocSizeOf)
+{
+    size_t usage = aMallocSizeOf(this);
+
+    usage += mCacheMap.SizeOfExcludingThis(aMallocSizeOf);
+    usage += mBindery.SizeOfExcludingThis(aMallocSizeOf);
+
+    return usage;
+}
+
