@@ -53,7 +53,7 @@ nsIndexedToHTML::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult) {
         return NS_ERROR_NO_AGGREGATION;
     
     nsIndexedToHTML* _s = new nsIndexedToHTML();
-    if (_s == nsnull)
+    if (_s == nullptr)
         return NS_ERROR_OUT_OF_MEMORY;
     
     rv = _s->QueryInterface(aIID, aResult);
@@ -156,7 +156,7 @@ nsIndexedToHTML::DoOnStartRequest(nsIRequest* request, nsISupports *aContext,
     rv = mParser->OnStartRequest(request, aContext);
     if (NS_FAILED(rv)) return rv;
 
-    nsCAutoString baseUri, titleUri;
+    nsAutoCString baseUri, titleUri;
     rv = uri->GetAsciiSpec(baseUri);
     if (NS_FAILED(rv)) return rv;
     titleUri = baseUri;
@@ -179,7 +179,7 @@ nsIndexedToHTML::DoOnStartRequest(nsIRequest* request, nsISupports *aContext,
         // This is done by the 300: line generation in ftp, but we don't use
         // that - see above
         
-        nsCAutoString pw;
+        nsAutoCString pw;
         rv = uri->GetPassword(pw);
         if (NS_FAILED(rv)) return rv;
         if (!pw.IsEmpty()) {
@@ -192,7 +192,7 @@ nsIndexedToHTML::DoOnStartRequest(nsIRequest* request, nsISupports *aContext,
              if (NS_FAILED(rv)) return rv;
         }
 
-        nsCAutoString path;
+        nsAutoCString path;
         rv = uri->GetPath(path);
         if (NS_FAILED(rv)) return rv;
 
@@ -207,7 +207,7 @@ nsIndexedToHTML::DoOnStartRequest(nsIRequest* request, nsISupports *aContext,
         if (NS_FAILED(rv)) return rv;
         file->SetFollowLinks(true);
         
-        nsCAutoString url;
+        nsAutoCString url;
         rv = net_GetURLSpecFromFile(file, url);
         if (NS_FAILED(rv)) return rv;
         baseUri.Assign(url);
@@ -226,7 +226,7 @@ nsIndexedToHTML::DoOnStartRequest(nsIRequest* request, nsISupports *aContext,
         NS_ENSURE_SUCCESS(rv, rv);
 
     } else if (NS_SUCCEEDED(uri->SchemeIs("jar", &isScheme)) && isScheme) {
-        nsCAutoString path;
+        nsAutoCString path;
         rv = uri->GetPath(path);
         if (NS_FAILED(rv)) return rv;
 
@@ -244,7 +244,7 @@ nsIndexedToHTML::DoOnStartRequest(nsIRequest* request, nsISupports *aContext,
     else {
         // default behavior for other protocols is to assume the channel's
         // URL references a directory ending in '/' -- fixup if necessary.
-        nsCAutoString path;
+        nsAutoCString path;
         rv = uri->GetPath(path);
         if (NS_FAILED(rv)) return rv;
         if (baseUri.Last() != '/') {
@@ -512,7 +512,7 @@ nsIndexedToHTML::DoOnStartRequest(nsIRequest* request, nsISupports *aContext,
     if (NS_FAILED(rv) && isSchemeFile) {
         nsCOMPtr<nsIPlatformCharset> platformCharset(do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv));
         NS_ENSURE_SUCCESS(rv, rv);
-        nsCAutoString charset;
+        nsAutoCString charset;
         rv = platformCharset->GetCharset(kPlatformCharsetSel_FileName, charset);
         NS_ENSURE_SUCCESS(rv, rv);
 
@@ -691,15 +691,15 @@ nsIndexedToHTML::FormatInputStream(nsIRequest* aRequest, nsISupports *aContext,
                                                           getter_AddRefs(mUnicodeEncoder));
         if (NS_SUCCEEDED(rv))
             rv = mUnicodeEncoder->SetOutputErrorBehavior(nsIUnicodeEncoder::kOnError_Replace, 
-                                                       nsnull, (PRUnichar)'?');
+                                                       nullptr, (PRUnichar)'?');
       }
     }
 
     // convert the data with unicode encoder
-    char *buffer = nsnull;
-    PRInt32 dstLength;
+    char *buffer = nullptr;
+    int32_t dstLength;
     if (NS_SUCCEEDED(rv)) {
-      PRInt32 unicharLength = aBuffer.Length();
+      int32_t unicharLength = aBuffer.Length();
       rv = mUnicodeEncoder->GetMaxLength(PromiseFlatString(aBuffer).get(), 
                                          unicharLength, &dstLength);
       if (NS_SUCCEEDED(rv)) {
@@ -709,7 +709,7 @@ nsIndexedToHTML::FormatInputStream(nsIRequest* aRequest, nsISupports *aContext,
         rv = mUnicodeEncoder->Convert(PromiseFlatString(aBuffer).get(), &unicharLength, 
                                       buffer, &dstLength);
         if (NS_SUCCEEDED(rv)) {
-          PRInt32 finLen = 0;
+          int32_t finLen = 0;
           rv = mUnicodeEncoder->Finish(buffer + dstLength, &finLen);
           if (NS_SUCCEEDED(rv))
             dstLength += finLen;
@@ -722,7 +722,7 @@ nsIndexedToHTML::FormatInputStream(nsIRequest* aRequest, nsISupports *aContext,
       rv = NS_OK;
       if (buffer) {
         nsMemory::Free(buffer);
-        buffer = nsnull;
+        buffer = nullptr;
       }
     }
 
@@ -748,8 +748,8 @@ NS_IMETHODIMP
 nsIndexedToHTML::OnDataAvailable(nsIRequest *aRequest,
                                  nsISupports *aCtxt,
                                  nsIInputStream* aInput,
-                                 PRUint32 aOffset,
-                                 PRUint32 aCount) {
+                                 uint64_t aOffset,
+                                 uint32_t aCount) {
     return mParser->OnDataAvailable(aRequest, aCtxt, aInput, aOffset, aCount);
 }
 
@@ -773,7 +773,7 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
 
     // The sort key is the name of the item, prepended by either 0, 1 or 2
     // in order to group items.
-    PRUint32 type;
+    uint32_t type;
     aIndex->GetType(&type);
     switch (type) {
         case nsIDirIndex::TYPE_SYMLINK:
@@ -820,7 +820,7 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
         descriptionAffix.Cut(0, descriptionAffix.Length() - 25);
         if (NS_IS_LOW_SURROGATE(descriptionAffix.First()))
             descriptionAffix.Cut(0, 1);
-        description.Truncate(NS_MIN<PRUint32>(71, description.Length() - 28));
+        description.Truncate(NS_MIN<uint32_t>(71, description.Length() - 28));
         if (NS_IS_HIGH_SURROGATE(description.Last()))
             description.Truncate(description.Length() - 1);
 
@@ -860,7 +860,7 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
     if (NS_FAILED(rv)) return rv;
 
     // need to escape links
-    nsCAutoString escapeBuf;
+    nsAutoCString escapeBuf;
 
     NS_ConvertUTF16toUTF8 utf8UnEscapeSpec(unEscapeSpec);
 
@@ -872,12 +872,12 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
     }
 
     // now minimally re-escape the location...
-    PRUint32 escFlags;
+    uint32_t escFlags;
     // for some protocols, we expect the location to be absolute.
     // if so, and if the location indeed appears to be a valid URI, then go
     // ahead and treat it like one.
     if (mExpectAbsLoc &&
-        NS_SUCCEEDED(net_ExtractURLScheme(utf8UnEscapeSpec, nsnull, nsnull, nsnull))) {
+        NS_SUCCEEDED(net_ExtractURLScheme(utf8UnEscapeSpec, nullptr, nullptr, nullptr))) {
         // escape as absolute 
         escFlags = esc_Forced | esc_OnlyASCII | esc_AlwaysCopy | esc_Minimal;
     }
@@ -902,7 +902,7 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
 
     if (type == nsIDirIndex::TYPE_FILE || type == nsIDirIndex::TYPE_UNKNOWN) {
         pushBuffer.AppendLiteral("<img src=\"moz-icon://");
-        PRInt32 lastDot = escapeBuf.RFindChar('.');
+        int32_t lastDot = escapeBuf.RFindChar('.');
         if (lastDot != kNotFound) {
             escapeBuf.Cut(0, lastDot);
             NS_ConvertUTF8toUTF16 utf16EscapeBuf(escapeBuf);
@@ -928,10 +928,10 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
     if (type == nsIDirIndex::TYPE_DIRECTORY || type == nsIDirIndex::TYPE_SYMLINK) {
         pushBuffer.AppendLiteral(">");
     } else {
-        PRInt64 size;
+        int64_t size;
         aIndex->GetSize(&size);
 
-        if (PRUint64(size) != LL_MAXUINT) {
+        if (uint64_t(size) != UINT64_MAX) {
             pushBuffer.AppendLiteral(" sortable-data=\"");
             pushBuffer.AppendInt(size);
             pushBuffer.AppendLiteral("\">");
@@ -951,17 +951,17 @@ nsIndexedToHTML::OnIndexAvailable(nsIRequest *aRequest,
         pushBuffer.AppendLiteral("></td>\n <td>");
     } else {
         pushBuffer.AppendLiteral(" sortable-data=\"");
-        pushBuffer.AppendInt(t);
+        pushBuffer.AppendInt(static_cast<int64_t>(t));
         pushBuffer.AppendLiteral("\">");
         nsAutoString formatted;
-        mDateTime->FormatPRTime(nsnull,
+        mDateTime->FormatPRTime(nullptr,
                                 kDateFormatShort,
                                 kTimeFormatNone,
                                 t,
                                 formatted);
         AppendNonAsciiToNCR(formatted, pushBuffer);
         pushBuffer.AppendLiteral("</td>\n <td>");
-        mDateTime->FormatPRTime(nsnull,
+        mDateTime->FormatPRTime(nullptr,
                                 kDateFormatNone,
                                 kTimeFormatSeconds,
                                 t,
@@ -991,12 +991,12 @@ nsIndexedToHTML::OnInformationAvailable(nsIRequest *aRequest,
     return FormatInputStream(aRequest, aCtxt, pushBuffer);
 }
 
-void nsIndexedToHTML::FormatSizeString(PRInt64 inSize, nsString& outSizeString)
+void nsIndexedToHTML::FormatSizeString(int64_t inSize, nsString& outSizeString)
 {
     outSizeString.Truncate();
-    if (inSize > PRInt64(0)) {
+    if (inSize > int64_t(0)) {
         // round up to the nearest Kilobyte
-        PRInt64  upperSize = (inSize + PRInt64(1023)) / PRInt64(1024);
+        int64_t  upperSize = (inSize + int64_t(1023)) / int64_t(1024);
         outSizeString.AppendInt(upperSize);
         outSizeString.AppendLiteral(" KB");
     }
