@@ -31,20 +31,24 @@ function BrowserRootActor(aConnection)
   this.conn = aConnection;
   this._tabActors = new WeakMap();
   this._tabActorPool = null;
-  this._actorFactories = null;
+  // A map of actor names to actor instances provided by extensions.
+  this._extraActors = {};
 
   this.onTabClosed = this.onTabClosed.bind(this);
   windowMediator.addListener(this);
 }
 
 BrowserRootActor.prototype = {
+
   /**
    * Return a 'hello' packet as specified by the Remote Debugging Protocol.
    */
   sayHello: function BRA_sayHello() {
-    return { from: "root",
-             applicationType: "browser",
-             traits: [] };
+    return {
+      from: "root",
+      applicationType: "browser",
+      traits: {}
+    };
   },
 
   /**
@@ -52,6 +56,7 @@ BrowserRootActor.prototype = {
    */
   disconnect: function BRA_disconnect() {
     windowMediator.removeListener(this);
+    this._extraActors = null;
 
     // We may have registered event listeners on browser windows to
     // watch for tab closes, remove those.
@@ -67,17 +72,25 @@ BrowserRootActor.prototype = {
   },
 
   /**
-   * Handles the listTabs request.  Builds a list of actors
-   * for the tabs running in the process.  The actors will survive
-   * until at least the next listTabs request.
+   * Handles the listTabs request.  Builds a list of actors for the tabs running
+   * in the process.  The actors will survive until at least the next listTabs
+   * request.
    */
   onListTabs: function BRA_onListTabs() {
-    // Get actors for all the currently-running tabs (reusing
-    // existing actors where applicable), and store them in
-    // an ActorPool.
+    // Get actors for all the currently-running tabs (reusing existing actors
+    // where applicable), and store them in an ActorPool.
 
     let actorPool = new ActorPool(this.conn);
-    let actorList = [];
+    let tabActorList = [];
+
+    // Get the chrome debugger actor.
+    let actor = this._chromeDebugger;
+    if (!actor) {
+      actor = new ChromeDebuggerActor(this);
+      actor.parentID = this.actorID;
+      this._chromeDebugger = actor;
+      actorPool.addActor(actor);
+    }
 
     // Walk over open browser windows.
     let e = windowMediator.getEnumerator("navigator:browser");
@@ -86,16 +99,16 @@ BrowserRootActor.prototype = {
     while (e.hasMoreElements()) {
       let win = e.getNext();
 
-      // Watch the window for tab closes so we can invalidate
-      // actors as needed.
+      // Watch the window for tab closes so we can invalidate actors as needed.
       this.watchWindow(win);
 
       // List the tabs in this browser.
       let selectedBrowser = win.getBrowser().selectedBrowser;
+
       let browsers = win.getBrowser().browsers;
       for each (let browser in browsers) {
         if (browser == selectedBrowser && win == top) {
-          selected = actorList.length;
+          selected = tabActorList.length;
         }
         let actor = this._tabActors.get(browser);
         if (!actor) {
@@ -104,23 +117,55 @@ BrowserRootActor.prototype = {
           this._tabActors.set(browser, actor);
         }
         actorPool.addActor(actor);
-        actorList.push(actor);
+        tabActorList.push(actor);
       }
     }
 
-    // Now drop the old actorID -> actor map.  Actors that still
-    // mattered were added to the new map, others will go
-    // away.
+    this._createExtraActors(DebuggerServer.globalActorFactories, actorPool);
+
+    // Now drop the old actorID -> actor map.  Actors that still mattered were
+    // added to the new map, others will go away.
     if (this._tabActorPool) {
       this.conn.removeActorPool(this._tabActorPool);
     }
     this._tabActorPool = actorPool;
     this.conn.addActorPool(this._tabActorPool);
 
-    return { "from": "root",
-             "selected": selected,
-             "tabs": [actor.grip()
-                      for each (actor in actorList)] };
+    let response = {
+      "from": "root",
+      "selected": selected,
+      "tabs": [actor.grip() for (actor of tabActorList)],
+      "chromeDebugger": this._chromeDebugger.actorID
+    };
+    this._appendExtraActors(response);
+    return response;
+  },
+
+  /**
+   * Adds dynamically-added actors from add-ons to the provided pool.
+   */
+  _createExtraActors: function BRA_createExtraActors(aFactories, aPool) {
+    // Walk over global actors added by extensions.
+    for (let name in aFactories) {
+      let actor = this._extraActors[name];
+      if (!actor) {
+        actor = aFactories[name].bind(null, this.conn, this);
+        actor.prototype = aFactories[name].prototype;
+        actor.parentID = this.actorID;
+        this._extraActors[name] = actor;
+      }
+      aPool.addActor(actor);
+    }
+  },
+
+  /**
+   * Appends the extra actors to the specified object.
+   */
+  _appendExtraActors: function BRA_appendExtraActors(aObject) {
+    for (let name in this._extraActors) {
+      let actor = this._extraActors[name];
+      aObject[name] = actor.actorID;
+    }
   },
 
   /**
@@ -128,20 +173,27 @@ BrowserRootActor.prototype = {
    * tab closures.
    */
   watchWindow: function BRA_watchWindow(aWindow) {
-    aWindow.getBrowser().tabContainer.addEventListener("TabClose",
-                                                       this.onTabClosed,
-                                                       false);
+    this.getTabContainer(aWindow).addEventListener("TabClose",
+                                                   this.onTabClosed,
+                                                   false);
   },
 
   /**
    * Stop watching a window for tab closes.
    */
   unwatchWindow: function BRA_unwatchWindow(aWindow) {
-    aWindow.getBrowser().tabContainer.removeEventListener("TabClose",
-                                                          this.onTabClosed);
+    this.getTabContainer(aWindow).removeEventListener("TabClose",
+                                                      this.onTabClosed);
     this.exitTabActor(aWindow);
   },
 
+  /**
+   * Return the tab container for the specified window.
+   */
+  getTabContainer: function BRA_getTabContainer(aWindow) {
+    return aWindow.getBrowser().tabContainer;
+  },
+
   /**
    * When a tab is closed, exit its tab actor.  The actor
    * will be dropped at the next listTabs request.
@@ -160,7 +212,46 @@ BrowserRootActor.prototype = {
     }
   },
 
-  // nsIWindowMediatorListener
+  // ChromeDebuggerActor hooks.
+
+  /**
+   * Add the specified actor to the default actor pool connection, in order to
+   * keep it alive as long as the server is. This is used by breakpoints in the
+   * thread and chrome debugger actors.
+   *
+   * @param actor aActor
+   *        The actor object.
+   */
+  addToParentPool: function BRA_addToParentPool(aActor) {
+    this.conn.addActor(aActor);
+  },
+
+  /**
+   * Remove the specified actor from the default actor pool.
+   *
+   * @param BreakpointActor aActor
+   *        The actor object.
+   */
+  removeFromParentPool: function BRA_removeFromParentPool(aActor) {
+    this.conn.removeActor(aActor);
+  },
+
+  /**
+   * Prepare to enter a nested event loop by disabling debuggee events.
+   */
+  preNest: function BRA_preNest() {
+    // Nothing to be done here for chrome windows.
+  },
+
+  /**
+   * Prepare to exit a nested event loop by enabling debuggee events.
+   */
+  postNest: function BRA_postNest(aNestData) {
+    // Nothing to be done here for chrome windows.
+  },
+
+  // nsIWindowMediatorListener.
+
   onWindowTitleChange: function BRA_onWindowTitleChange(aWindow, aTitle) { },
   onOpenWindow: function BRA_onOpenWindow(aWindow) { },
   onCloseWindow: function BRA_onCloseWindow(aWindow) {
@@ -196,7 +287,12 @@ function BrowserTabActor(aConnection, aBrowser, aTabBrowser)
   this.conn = aConnection;
   this._browser = aBrowser;
   this._tabbrowser = aTabBrowser;
+  this._tabActorPool = null;
+  // A map of actor names to actor instances provided by extensions.
+  this._extraActors = {};
 
+  this._createExtraActors = BrowserRootActor.prototype._createExtraActors.bind(this);
+  this._appendExtraActors = BrowserRootActor.prototype._appendExtraActors.bind(this);
   this._onWindowCreated = this.onWindowCreated.bind(this);
 }
 
@@ -218,26 +314,28 @@ BrowserTabActor.prototype = {
   _pendingNavigation: null,
 
   /**
-   * Add the specified breakpoint to the default actor pool connection, in order
-   * to be alive as long as the server is.
+   * Add the specified actor to the default actor pool connection, in order to
+   * keep it alive as long as the server is. This is used by breakpoints in the
+   * thread actor.
    *
-   * @param BreakpointActor aActor
+   * @param actor aActor
    *        The actor object.
    */
-  addToBreakpointPool: function BTA_addToBreakpointPool(aActor) {
+  addToParentPool: function BTA_addToParentPool(aActor) {
     this.conn.addActor(aActor);
   },
 
   /**
-   * Remove the specified breakpint from the default actor pool.
+   * Remove the specified actor from the default actor pool.
    *
-   * @param string aActor
-   *        The actor ID.
+   * @param BreakpointActor aActor
+   *        The actor object.
    */
-  removeFromBreakpointPool: function BTA_removeFromBreakpointPool(aActor) {
+  removeFromParentPool: function BTA_removeFromParentPool(aActor) {
     this.conn.removeActor(aActor);
   },
 
+  // A constant prefix that will be used to form the actor ID by the server.
   actorPrefix: "tab",
 
   grip: function BTA_grip() {
@@ -245,9 +343,31 @@ BrowserTabActor.prototype = {
                "grip() shouldn't be called on exited browser actor.");
     dbg_assert(this.actorID,
                "tab should have an actorID.");
-    return { actor: this.actorID,
-             title: this.browser.contentTitle,
-             url: this.browser.currentURI.spec }
+
+    let title = this.browser.contentTitle;
+    // If contentTitle is empty (e.g. on a not-yet-restored tab), but there is a
+    // tabbrowser (i.e. desktop Firefox, but not Fennec), we can use the label
+    // as the title.
+    if (!title && this._tabbrowser) {
+      title = this._tabbrowser
+                  ._getTabForContentWindow(this.browser.contentWindow).label;
+    }
+    let response = {
+      actor: this.actorID,
+      title: title,
+      url: this.browser.currentURI.spec
+    };
+
+    // Walk over tab actors added by extensions and add them to a new ActorPool.
+    let actorPool = new ActorPool(this.conn);
+    this._createExtraActors(DebuggerServer.tabActorFactories, actorPool);
+    if (!actorPool.isEmpty()) {
+      this._tabActorPool = actorPool;
+      this.conn.addActorPool(this._tabActorPool);
+    }
+
+    this._appendExtraActors(response);
+    return response;
   },
 
   /**
@@ -259,6 +379,7 @@ BrowserTabActor.prototype = {
     if (this._progressListener) {
       this._progressListener.destroy();
     }
+    this._extraActors = null;
   },
 
   /**
@@ -301,7 +422,9 @@ BrowserTabActor.prototype = {
     // Watch for globals being created in this tab.
     this.browser.addEventListener("DOMWindowCreated", this._onWindowCreated, true);
     this.browser.addEventListener("pageshow", this._onWindowCreated, true);
-    this._progressListener = new DebuggerProgressListener(this);
+    if (this._tabbrowser) {
+      this._progressListener = new DebuggerProgressListener(this);
+    }
 
     this._attached = true;
   },
@@ -316,22 +439,10 @@ BrowserTabActor.prototype = {
     this._contextPool = new ActorPool(this.conn);
     this.conn.addActorPool(this._contextPool);
 
-    this.threadActor = new ThreadActor(this);
-    this._addDebuggees(this.browser.contentWindow.wrappedJSObject);
+    this.threadActor = new ThreadActor(this, this.browser.contentWindow.wrappedJSObject);
     this._contextPool.addActor(this.threadActor);
   },
 
-  /**
-   * Add the provided window and all windows in its frame tree as debuggees.
-   */
-  _addDebuggees: function BTA__addDebuggees(aWindow) {
-    this.threadActor.addDebuggee(aWindow);
-    let frames = aWindow.frames;
-    for (let i = 0; i < frames.length; i++) {
-      this._addDebuggees(frames[i]);
-    }
-  },
-
   /**
    * Exits the current thread actor and removes the context-lifetime actor pool.
    * The content window is no longer being debugged after this call.
@@ -361,6 +472,10 @@ BrowserTabActor.prototype = {
     // Shut down actors that belong to this tab's pool.
     this.conn.removeActorPool(this._tabPool);
     this._tabPool = null;
+    if (this._tabActorPool) {
+      this.conn.removeActorPool(this._tabActorPool);
+      this._tabActorPool = null;
+    }
 
     this._attached = false;
   },
@@ -438,7 +553,9 @@ BrowserTabActor.prototype = {
       }
       if (this._attached) {
         this.threadActor.clearDebuggees();
-        this.threadActor.dbg.enabled = true;
+        if (this.threadActor.dbg) {
+          this.threadActor.dbg.enabled = true;
+        }
         if (this._progressListener) {
           delete this._progressListener._needsTabNavigated;
         }
@@ -448,7 +565,10 @@ BrowserTabActor.prototype = {
     }
 
     if (this._attached) {
-      this._addDebuggees(evt.target.defaultView.wrappedJSObject);
+      this.threadActor.global = evt.target.defaultView.wrappedJSObject;
+      if (this.threadActor.attached) {
+        this.threadActor.findGlobals();
+      }
     }
   }
 };
@@ -520,26 +640,10 @@ DebuggerProgressListener.prototype = {
    * Destroy the progress listener instance.
    */
   destroy: function DPL_destroy() {
-    this._tabActor._tabbrowser.removeProgressListener(this);
+    if (this._tabActor._tabbrowser.removeProgressListener) {
+      this._tabActor._tabbrowser.removeProgressListener(this);
+    }
     this._tabActor._progressListener = null;
     this._tabActor = null;
   }
 };
-
-/**
- * Registers handlers for new request types defined dynamically. This is used
- * for example by add-ons to augment the functionality of the tab actor.
- *
- * @param aName string
- *        The name of the new request type.
- * @param aFunction function
- *        The handler for this request type.
- */
-DebuggerServer.addTabRequest = function DS_addTabRequest(aName, aFunction) {
-  BrowserTabActor.prototype.requestTypes[aName] = function(aRequest) {
-    if (!this.attached) {
-      return { error: "wrongState" };
-    }
-    return aFunction(this, aRequest);
-  }
-};
